/* 
 * $RCSfile: gj_config.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/08/19 22:54:00 $
 * 
 * $Revision: 1.11 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <locale.h>

#include <glib.h>

#include "common.h"

#include "gj_config.h"
#include "gj_config_handler.h"

struct 
{
  gchar *cs_server;
  gint cs_port;
  gchar *cs_username;
  gchar *cs_password;

  gchar *cs_nickname;
  gchar *cs_resource;
  gint8 cs_priority;

  gboolean cs_save_password;
  gboolean cs_use_plain_text_password;
  gboolean cs_use_ssl;

  gboolean cs_auto_connect_on_startup;
  gboolean cs_auto_reconnect_on_disconnect;

  gboolean mpf_hide_offline_contacts;
  gboolean mpf_hide_offline_group;
  gboolean mpf_hide_agents;
  gboolean mpf_hide_contact_presence_details;

  gboolean mpf_double_click_for_chat;

  gboolean mpf_close_previous_message_window_on_reply;
  gboolean mpf_raise_window_on_new_message;
  gboolean mpf_hide_time;
  gboolean mpf_reverse_text_direction;

  gboolean mpf_use_spell_checker;

  gint mpf_auto_away_after;
  gint mpf_auto_xa_after;
  gchar *mpf_auto_away_status;
  gchar *mpf_auto_xa_status;

  gboolean mpf_sound_enabled;
  gboolean mpf_sound_on_user_online;
  gboolean mpf_sound_on_user_offline;
  gboolean mpf_sound_on_receive_message;
  gboolean mpf_sound_on_receive_chat;
  gboolean mpf_sound_on_receive_url;
  gboolean mpf_sound_on_receive_error;
  gboolean mpf_sound_beep_if_no_file;

  gchar *mpf_sound_file_on_user_online;
  gchar *mpf_sound_file_on_user_offline;
  gchar *mpf_sound_file_on_receive_message;
  gchar *mpf_sound_file_on_receive_chat;
  gchar *mpf_sound_file_on_receive_url;
  gchar *mpf_sound_file_on_receive_error;

  gboolean mpf_use_docklet;

  gchar *mpf_log_location;
  gchar *mpf_history_location;
  gboolean mpf_use_history;

} cfg;

gchar *config_filename = NULL;

gboolean gj_config_init(gchar *filename)
{
  if(filename == NULL)
    {
      g_message("gj_config_init: filename not specified, using default:'%s'", FILE_CONFIG);
      filename = FILE_CONFIG;
    }
  else
      g_message("gj_config_init: using filename:'%s'", filename);

  /* set config filename */
  config_filename = g_strdup(filename);

  /* set defaults */
  cfg.cs_server = NULL;
  cfg.cs_port = 5222;
  cfg.cs_username = NULL;
  cfg.cs_password = NULL;

  cfg.cs_nickname = NULL; 
  cfg.cs_resource = NULL;
  cfg.cs_priority = 10;

  cfg.cs_save_password = FALSE;
  cfg.cs_use_plain_text_password = FALSE;
  cfg.cs_use_ssl = FALSE;
	
  cfg.cs_auto_connect_on_startup = FALSE;
  cfg.cs_auto_reconnect_on_disconnect = FALSE;
	
  cfg.mpf_hide_offline_contacts = TRUE;
  cfg.mpf_hide_offline_group = FALSE;
  cfg.mpf_hide_agents = FALSE;
  cfg.mpf_hide_contact_presence_details = TRUE;

  cfg.mpf_double_click_for_chat = TRUE;

  cfg.mpf_close_previous_message_window_on_reply = TRUE;
  cfg.mpf_raise_window_on_new_message = FALSE;
  cfg.mpf_hide_time = FALSE;
  cfg.mpf_reverse_text_direction = FALSE;

  cfg.mpf_use_spell_checker = FALSE;

  cfg.mpf_auto_away_after = 5;
  cfg.mpf_auto_xa_after = 15;
  cfg.mpf_auto_away_status = NULL;
  cfg.mpf_auto_xa_status = NULL;

  cfg.mpf_sound_enabled = FALSE;
  cfg.mpf_sound_on_user_online = FALSE;
  cfg.mpf_sound_on_user_offline = FALSE;
  cfg.mpf_sound_on_receive_message = FALSE;
  cfg.mpf_sound_on_receive_chat = FALSE;
  cfg.mpf_sound_on_receive_url = FALSE;
  cfg.mpf_sound_on_receive_error = FALSE;
  cfg.mpf_sound_beep_if_no_file = FALSE;
	
  cfg.mpf_sound_file_on_user_online = NULL;
  cfg.mpf_sound_file_on_user_offline = NULL;
  cfg.mpf_sound_file_on_receive_message = NULL;
  cfg.mpf_sound_file_on_receive_chat = NULL;
  cfg.mpf_sound_file_on_receive_url = NULL;
  cfg.mpf_sound_file_on_receive_error = NULL;

  cfg.mpf_use_docklet = FALSE;

  cfg.mpf_log_location = NULL;
  cfg.mpf_history_location = NULL;
  cfg.mpf_use_history = TRUE;
  
  return TRUE;
}

gboolean gj_config_term()
{
  if(config_filename != NULL)
    {
      g_free(config_filename);
      config_filename = NULL;
    }

  /* set defaults */
  if(cfg.cs_server != NULL)
  {
    g_free(cfg.cs_server);
    cfg.cs_server = NULL;
  }

  cfg.cs_port = 5222;


  if(cfg.cs_username != NULL)
    {
      g_free(cfg.cs_username);
      cfg.cs_username = NULL;
    }

  if(cfg.cs_password != NULL)
    {
      g_free(cfg.cs_password);
      cfg.cs_password = NULL;
    }

  if(cfg.cs_nickname != NULL)
    {
      g_free(cfg.cs_nickname);
      cfg.cs_nickname = NULL; 
    }

  if(cfg.cs_resource != NULL)
    {
      g_free(cfg.cs_resource);
      cfg.cs_resource = NULL;
    }

  cfg.cs_priority = 10;


  cfg.cs_save_password = FALSE;
  cfg.cs_use_plain_text_password = FALSE;
  cfg.cs_use_ssl = FALSE;
	
  cfg.cs_auto_connect_on_startup = FALSE;
  cfg.cs_auto_reconnect_on_disconnect = FALSE;
	
  cfg.mpf_hide_offline_contacts = TRUE;
  cfg.mpf_hide_offline_group = FALSE;
  cfg.mpf_hide_agents = FALSE;
  cfg.mpf_hide_contact_presence_details = TRUE;

  cfg.mpf_double_click_for_chat = TRUE;

  cfg.mpf_close_previous_message_window_on_reply = TRUE;
  cfg.mpf_raise_window_on_new_message = TRUE;
  cfg.mpf_hide_time = FALSE;
  cfg.mpf_reverse_text_direction = FALSE;

  cfg.mpf_use_spell_checker = FALSE;

  cfg.mpf_auto_away_after = 5;
  cfg.mpf_auto_xa_after = 15;
  cfg.mpf_auto_away_status = NULL;
  cfg.mpf_auto_xa_status = NULL;

  cfg.mpf_sound_enabled = FALSE;
  cfg.mpf_sound_on_user_online = FALSE;
  cfg.mpf_sound_on_user_offline = FALSE;
  cfg.mpf_sound_on_receive_message = FALSE;
  cfg.mpf_sound_on_receive_chat = FALSE;
  cfg.mpf_sound_on_receive_url = FALSE;
  cfg.mpf_sound_on_receive_error = FALSE;
  cfg.mpf_sound_beep_if_no_file = FALSE;
	
  if(cfg.mpf_sound_file_on_user_online != NULL)
    {
      g_free(cfg.mpf_sound_file_on_user_online);
      cfg.mpf_sound_file_on_user_online = NULL;
    }

  if(cfg.mpf_sound_file_on_user_offline != NULL)
    {
      g_free(cfg.mpf_sound_file_on_user_offline);
      cfg.mpf_sound_file_on_user_offline = NULL;
    }

  if(cfg.mpf_sound_file_on_receive_message != NULL)
    {
      g_free(cfg.mpf_sound_file_on_receive_message);
      cfg.mpf_sound_file_on_receive_message = NULL;
    }

  if(cfg.mpf_sound_file_on_receive_chat != NULL)
    {
      g_free(cfg.mpf_sound_file_on_receive_chat);
      cfg.mpf_sound_file_on_receive_chat = NULL;
    }

  if(cfg.mpf_sound_file_on_receive_url != NULL) 
    {
      g_free(cfg.mpf_sound_file_on_receive_url);
      cfg.mpf_sound_file_on_receive_url = NULL;
    }

  if(cfg.mpf_sound_file_on_receive_error != NULL)
    {
      g_free(cfg.mpf_sound_file_on_receive_error);
      cfg.mpf_sound_file_on_receive_error = NULL;
    }

  cfg.mpf_use_docklet = FALSE;

  if(cfg.mpf_log_location != NULL)
    {
      g_free(cfg.mpf_log_location);
      cfg.mpf_log_location = NULL;
    }

  if(cfg.mpf_history_location != NULL)
    {
      g_free(cfg.mpf_history_location);
      cfg.mpf_history_location = NULL;
    }

  cfg.mpf_use_history = TRUE;
  
  return TRUE;
}

gboolean gj_config_read()
{
  configFile *cf;

  if(config_filename == NULL || strlen(config_filename) < 1)
    {
      g_warning("gj_config_read: filename was NULL or len < 1");
      return FALSE;
    }

  /*  memset(&cfg, 0, sizeof (Config));*/

  /* get file */
  if((cf = gj_config_handler_open_file(config_filename)) == FALSE)
    {
      g_warning("gj_config_read: could not open config file:'%s'", config_filename);

      /* set defaults... */
      cfg.cs_server = g_strdup_printf("jabber.org");
      cfg.cs_username = NULL;
      cfg.cs_password = NULL;
      cfg.cs_resource = g_strdup_printf("gnome-jabber");
      cfg.cs_nickname = NULL;  
      
      cfg.mpf_auto_away_status = g_strdup_printf("Automatically Away");
      cfg.mpf_auto_xa_status = g_strdup_printf("Automatically Extended Away");

/*       cfg.mpf_log_location = g_get_current_dir(); */

      return FALSE;
    }

  gj_config_handler_read_string(cf, CONFIG_SECT_CS, CONFIG_CS_SERVER, &cfg.cs_server);
  gj_config_handler_read_int(cf, CONFIG_SECT_CS, CONFIG_CS_PORT, &cfg.cs_port);
  gj_config_handler_read_string(cf, CONFIG_SECT_CS, CONFIG_CS_USERNAME, &cfg.cs_username);
  gj_config_handler_read_string(cf, CONFIG_SECT_CS, CONFIG_CS_PASSWORD, &cfg.cs_password);

  gj_config_handler_read_string(cf, CONFIG_SECT_CS, CONFIG_CS_NICKNAME, &cfg.cs_nickname);
  gj_config_handler_read_string(cf, CONFIG_SECT_CS, CONFIG_CS_RESOURCE, &cfg.cs_resource);
  gj_config_handler_read_int8(cf, CONFIG_SECT_CS, CONFIG_CS_PRIORITY, &cfg.cs_priority);

  gj_config_handler_read_boolean(cf, CONFIG_SECT_CS, CONFIG_CS_SAVE_PASSWORD, &cfg.cs_save_password);
  gj_config_handler_read_boolean(cf, CONFIG_SECT_CS, CONFIG_CS_USE_PLAIN_TEXT_PASSWORD, &cfg.cs_use_plain_text_password);
  gj_config_handler_read_boolean(cf, CONFIG_SECT_CS, CONFIG_CS_USE_SSL, &cfg.cs_use_ssl);

  gj_config_handler_read_boolean(cf, CONFIG_SECT_CS, CONFIG_CS_AUTO_CONNECT_ON_STARTUP, &cfg.cs_auto_connect_on_startup);
  gj_config_handler_read_boolean(cf, CONFIG_SECT_CS, CONFIG_CS_AUTO_RECONNECT_ON_DISCONNECT, &cfg.cs_auto_reconnect_on_disconnect);

  gj_config_handler_read_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_HIDE_OFFLINE_CONTACTS, &cfg.mpf_hide_offline_contacts);
  gj_config_handler_read_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_HIDE_OFFLINE_GROUP, &cfg.mpf_hide_offline_group);
  gj_config_handler_read_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_HIDE_AGENTS, &cfg.mpf_hide_agents);
  gj_config_handler_read_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_HIDE_CONTACT_PRESENCE_DETAILS, &cfg.mpf_hide_contact_presence_details);

  gj_config_handler_read_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_DOUBLE_CLICK_FOR_CHAT, &cfg.mpf_double_click_for_chat);

  gj_config_handler_read_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_CLOSE_PREVIOUS_MESSAGE_WINDOW_ON_REPLY, &cfg.mpf_close_previous_message_window_on_reply);
  gj_config_handler_read_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_RAISE_WINDOW_ON_NEW_MESSAGE, &cfg.mpf_raise_window_on_new_message);
  gj_config_handler_read_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_HIDE_TIME, &cfg.mpf_hide_time);
  gj_config_handler_read_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_REVERSE_TEXT_DIRECTION, &cfg.mpf_reverse_text_direction);

  gj_config_handler_read_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_USE_SPELL_CHECKER, &cfg.mpf_use_spell_checker);

  gj_config_handler_read_int(cf, CONFIG_SECT_MPF, CONFIG_MPF_AUTO_AWAY_AFTER, &cfg.mpf_auto_away_after);
  gj_config_handler_read_int(cf, CONFIG_SECT_MPF, CONFIG_MPF_AUTO_XA_AFTER, &cfg.mpf_auto_xa_after);
  gj_config_handler_read_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_AUTO_AWAY_STATUS, &cfg.mpf_auto_away_status);
  gj_config_handler_read_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_AUTO_XA_STATUS, &cfg.mpf_auto_xa_status);

  gj_config_handler_read_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_ON_USER_ONLINE, &cfg.mpf_sound_on_user_online);
  gj_config_handler_read_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_ON_USER_OFFLINE, &cfg.mpf_sound_on_user_offline);
  gj_config_handler_read_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_ON_RECEIVE_MESSAGE, &cfg.mpf_sound_on_receive_message);
  gj_config_handler_read_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_ON_RECEIVE_CHAT, &cfg.mpf_sound_on_receive_chat);
  gj_config_handler_read_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_ON_RECEIVE_URL, &cfg.mpf_sound_on_receive_url);
  gj_config_handler_read_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_ON_RECEIVE_ERROR, &cfg.mpf_sound_on_receive_error);
  gj_config_handler_read_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_FILE_ON_USER_ONLINE, &cfg.mpf_sound_file_on_user_online);
  gj_config_handler_read_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_FILE_ON_USER_OFFLINE, &cfg.mpf_sound_file_on_user_offline);
  gj_config_handler_read_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_FILE_ON_RECEIVE_MESSAGE, &cfg.mpf_sound_file_on_receive_message);
  gj_config_handler_read_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_FILE_ON_RECEIVE_CHAT, &cfg.mpf_sound_file_on_receive_chat);
  gj_config_handler_read_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_FILE_ON_RECEIVE_URL, &cfg.mpf_sound_file_on_receive_url);
  gj_config_handler_read_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_FILE_ON_RECEIVE_ERROR, &cfg.mpf_sound_file_on_receive_error);

  gj_config_handler_read_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_USE_DOCKLET, &cfg.mpf_use_docklet);

  gj_config_handler_read_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_LOG_LOCATION, &cfg.mpf_log_location);
  gj_config_handler_read_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_HISTORY_LOCATION, &cfg.mpf_history_location);
  gj_config_handler_read_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_USE_HISTORY, &cfg.mpf_use_history);
  
  /* clean up */
  gj_config_handler_free(cf);

  return TRUE;
}

gboolean gj_config_write()
{
  configFile *cf;

  if(config_filename == NULL || strlen(config_filename) < 1)
    {
      g_warning("gj_config_write: filename was NULL or len < 1");
      return FALSE;
    }

  if((cf = gj_config_handler_open_file(config_filename)) == FALSE)
    {
      g_warning("gj_config_write: could not open config file, creating...");

      if((cf = gj_config_handler_new()) == FALSE)
	{
	  g_warning("gj_config_write: could not create new config file.");
	  return FALSE;
	}
    }

  /* set values */
  gj_config_handler_write_string(cf, CONFIG_SECT_CS, CONFIG_CS_SERVER, cfg.cs_server);
  gj_config_handler_write_int(cf, CONFIG_SECT_CS, CONFIG_CS_PORT, cfg.cs_port);
  gj_config_handler_write_string(cf, CONFIG_SECT_CS, CONFIG_CS_USERNAME, cfg.cs_username);
  gj_config_handler_write_string(cf, CONFIG_SECT_CS, CONFIG_CS_PASSWORD, cfg.cs_password);

  gj_config_handler_write_string(cf, CONFIG_SECT_CS, CONFIG_CS_NICKNAME, cfg.cs_nickname);
  gj_config_handler_write_string(cf, CONFIG_SECT_CS, CONFIG_CS_RESOURCE, cfg.cs_resource);
  gj_config_handler_write_int8(cf, CONFIG_SECT_CS, CONFIG_CS_PRIORITY, cfg.cs_priority);


  gj_config_handler_write_boolean(cf, CONFIG_SECT_CS, CONFIG_CS_SAVE_PASSWORD, cfg.cs_save_password);
  gj_config_handler_write_boolean(cf, CONFIG_SECT_CS, CONFIG_CS_USE_PLAIN_TEXT_PASSWORD, cfg.cs_use_plain_text_password);
  gj_config_handler_write_boolean(cf, CONFIG_SECT_CS, CONFIG_CS_USE_SSL, cfg.cs_use_ssl);

  gj_config_handler_write_boolean(cf, CONFIG_SECT_CS, CONFIG_CS_AUTO_CONNECT_ON_STARTUP, cfg.cs_auto_connect_on_startup);
  gj_config_handler_write_boolean(cf, CONFIG_SECT_CS, CONFIG_CS_AUTO_RECONNECT_ON_DISCONNECT, cfg.cs_auto_reconnect_on_disconnect);


  gj_config_handler_write_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_HIDE_OFFLINE_CONTACTS, cfg.mpf_hide_offline_contacts);
  gj_config_handler_write_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_HIDE_OFFLINE_GROUP, cfg.mpf_hide_offline_group);
  gj_config_handler_write_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_HIDE_AGENTS, cfg.mpf_hide_agents);
  gj_config_handler_write_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_HIDE_CONTACT_PRESENCE_DETAILS, cfg.mpf_hide_contact_presence_details);

  gj_config_handler_write_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_DOUBLE_CLICK_FOR_CHAT, cfg.mpf_double_click_for_chat);

  gj_config_handler_write_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_CLOSE_PREVIOUS_MESSAGE_WINDOW_ON_REPLY, cfg.mpf_close_previous_message_window_on_reply);
  gj_config_handler_write_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_RAISE_WINDOW_ON_NEW_MESSAGE, cfg.mpf_raise_window_on_new_message);
  gj_config_handler_write_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_HIDE_TIME, cfg.mpf_hide_time);
  gj_config_handler_write_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_REVERSE_TEXT_DIRECTION, cfg.mpf_reverse_text_direction);

  gj_config_handler_write_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_USE_SPELL_CHECKER, cfg.mpf_use_spell_checker);

  gj_config_handler_write_int(cf, CONFIG_SECT_MPF, CONFIG_MPF_AUTO_AWAY_AFTER, cfg.mpf_auto_away_after);
  gj_config_handler_write_int(cf, CONFIG_SECT_MPF, CONFIG_MPF_AUTO_XA_AFTER, cfg.mpf_auto_xa_after);
  gj_config_handler_write_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_AUTO_AWAY_STATUS, cfg.mpf_auto_away_status);
  gj_config_handler_write_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_AUTO_XA_STATUS, cfg.mpf_auto_xa_status);

  gj_config_handler_write_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_AUTO_XA_STATUS, cfg.mpf_auto_xa_status);

  gj_config_handler_write_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_ON_USER_ONLINE, cfg.mpf_sound_on_user_online);
  gj_config_handler_write_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_ON_USER_OFFLINE, cfg.mpf_sound_on_user_offline);
  gj_config_handler_write_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_ON_RECEIVE_MESSAGE, cfg.mpf_sound_on_receive_message);
  gj_config_handler_write_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_ON_RECEIVE_CHAT, cfg.mpf_sound_on_receive_chat);
  gj_config_handler_write_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_ON_RECEIVE_URL, cfg.mpf_sound_on_receive_url);
  gj_config_handler_write_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_ON_RECEIVE_ERROR, cfg.mpf_sound_on_receive_error);
  gj_config_handler_write_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_FILE_ON_USER_ONLINE, cfg.mpf_sound_file_on_user_online);
  gj_config_handler_write_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_FILE_ON_USER_OFFLINE, cfg.mpf_sound_file_on_user_offline);
  gj_config_handler_write_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_FILE_ON_RECEIVE_MESSAGE, cfg.mpf_sound_file_on_receive_message);
  gj_config_handler_write_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_FILE_ON_RECEIVE_CHAT, cfg.mpf_sound_file_on_receive_chat);
  gj_config_handler_write_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_FILE_ON_RECEIVE_URL, cfg.mpf_sound_file_on_receive_url);
  gj_config_handler_write_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_SOUND_FILE_ON_RECEIVE_ERROR, cfg.mpf_sound_file_on_receive_error);

  gj_config_handler_write_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_USE_DOCKLET, cfg.mpf_use_docklet);

  gj_config_handler_write_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_LOG_LOCATION, cfg.mpf_log_location);
  gj_config_handler_write_string(cf, CONFIG_SECT_MPF, CONFIG_MPF_HISTORY_LOCATION, cfg.mpf_history_location);
  gj_config_handler_write_boolean(cf, CONFIG_SECT_MPF, CONFIG_MPF_USE_HISTORY, cfg.mpf_use_history);

  /* write file */
  gj_config_handler_write_file(cf, config_filename);

  /* clean up */
  gj_config_handler_free(cf);

  return TRUE;
}

/*
 * GETS 
 */
gchar *gj_config_get_cs_server() { return cfg.cs_server; }
guint gj_config_get_cs_port() { return cfg.cs_port; }
gchar *gj_config_get_cs_username() { return cfg.cs_username; }
gchar *gj_config_get_cs_password() { return cfg.cs_password; }

gchar *gj_config_get_cs_nickname() { return cfg.cs_nickname; }
gchar *gj_config_get_cs_resource() { return cfg.cs_resource; }
gint8 gj_config_get_cs_priority() { return cfg.cs_priority; }

gboolean gj_config_get_cs_save_password() { return cfg.cs_save_password; }
gboolean gj_config_get_cs_use_plain_text_password() { return cfg.cs_use_plain_text_password; }
gboolean gj_config_get_cs_use_ssl() { return cfg.cs_use_ssl; }

gboolean gj_config_get_cs_auto_connect_on_startup() { return cfg.cs_auto_connect_on_startup; } 
gboolean gj_config_get_cs_auto_reconnect_on_disconnect() { return cfg.cs_auto_reconnect_on_disconnect; } 

gboolean gj_config_get_mpf_hide_offline_contacts() { return cfg.mpf_hide_offline_contacts ; } 
gboolean gj_config_get_mpf_hide_offline_group() { return cfg.mpf_hide_offline_group ; } 
gboolean gj_config_get_mpf_hide_agents() { return cfg.mpf_hide_agents ; } 
gboolean gj_config_get_mpf_hide_contact_presence_details() { return cfg.mpf_hide_contact_presence_details ; } 

gboolean gj_config_get_mpf_double_click_for_chat() { return cfg.mpf_double_click_for_chat ; } 

gboolean gj_config_get_mpf_close_previous_message_window_on_reply() { return cfg.mpf_close_previous_message_window_on_reply ; } 
gboolean gj_config_get_mpf_raise_window_on_new_message() { return cfg.mpf_raise_window_on_new_message ; } 
gboolean gj_config_get_mpf_hide_time() { return cfg.mpf_hide_time ; } 
gboolean gj_config_get_mpf_reverse_text_direction() { return cfg.mpf_reverse_text_direction ; } 

gboolean gj_config_get_mpf_use_spell_checker() { return cfg.mpf_use_spell_checker ; } 

guint gj_config_get_mpf_auto_away_after() { return cfg.mpf_auto_away_after ; } 
guint gj_config_get_mpf_auto_xa_after() { return cfg.mpf_auto_xa_after ; } 
gchar *gj_config_get_mpf_auto_away_status() { return cfg.mpf_auto_away_status; }
gchar *gj_config_get_mpf_auto_xa_status() { return cfg.mpf_auto_xa_status; }

gboolean gj_config_get_mpf_sound_on_user_online() { return cfg.mpf_sound_on_user_online; }
gboolean gj_config_get_mpf_sound_on_user_offline() { return cfg.mpf_sound_on_user_offline; }
gboolean gj_config_get_mpf_sound_on_receive_message() { return cfg.mpf_sound_on_receive_message; }
gboolean gj_config_get_mpf_sound_on_receive_chat() { return cfg.mpf_sound_on_receive_chat; }
gboolean gj_config_get_mpf_sound_on_receive_url() { return cfg.mpf_sound_on_receive_url; }
gboolean gj_config_get_mpf_sound_on_receive_error() { return cfg.mpf_sound_on_receive_error; }
gchar *gj_config_get_mpf_sound_file_on_user_online() { return cfg.mpf_sound_file_on_user_online; }
gchar *gj_config_get_mpf_sound_file_on_user_offline() { return cfg.mpf_sound_file_on_user_offline; }
gchar *gj_config_get_mpf_sound_file_on_receive_message() { return cfg.mpf_sound_file_on_receive_message; }
gchar *gj_config_get_mpf_sound_file_on_receive_chat() { return cfg.mpf_sound_file_on_receive_chat; }
gchar *gj_config_get_mpf_sound_file_on_receive_url() { return cfg.mpf_sound_file_on_receive_url; }
gchar *gj_config_get_mpf_sound_file_on_receive_error() { return cfg.mpf_sound_file_on_receive_error; }

gboolean gj_config_get_mpf_use_docklet() { return cfg.mpf_use_docklet; }

gchar *gj_config_get_mpf_log_location() { return cfg.mpf_log_location; }
gchar *gj_config_get_mpf_history_location() { return cfg.mpf_history_location; }
gboolean gj_config_get_mpf_use_history() { return cfg.mpf_use_history; }


/*
 * SETS
 */
gboolean gj_config_set_cs_server(gchar *string) 
{
  if(string == NULL) 
    {
      g_warning("gj_config_set_cs_server: server was NULL");
      return FALSE;
    }

  if(cfg.cs_server != NULL)
    {
      g_free(cfg.cs_server);
      cfg.cs_server = NULL;
    }

  cfg.cs_server = g_strdup(string);

  return TRUE;
}

gboolean gj_config_set_cs_port(guint integer) 
{
  if(integer < 1 || integer > 65535)
    {
      g_warning("gj_config_set_cs_port: port value:%d was out of range (1 to 64k)", integer);
      return FALSE;
    }

  cfg.cs_port = integer;

  return TRUE;
}

gboolean gj_config_set_cs_username(gchar *string) 
{
  if(string == NULL) 
    {
      g_warning("gj_config_set_cs_username: username was NULL");
      return FALSE;
    }

  if(cfg.cs_username != NULL)
    {
      g_free(cfg.cs_username);
      cfg.cs_username = NULL;
    }

  cfg.cs_username = g_strdup(string);

  return TRUE;
}

gboolean gj_config_set_cs_password(gchar *string) 
{
  if(string == NULL) 
    {
      g_warning("gj_config_set_cs_password: password was NULL");
      return FALSE;
    }

  if(cfg.cs_password != NULL)
    {
      g_free(cfg.cs_password);
      cfg.cs_password = NULL;
    }

  cfg.cs_password = g_strdup(string);

  return TRUE;
}

gboolean gj_config_set_cs_nickname(gchar *string) 
{
  if(string == NULL) 
    {
      g_warning("gj_config_set_cs_nickname: nickname was NULL");
      return FALSE;
    }

  if(cfg.cs_nickname != NULL)
    {
      g_free(cfg.cs_nickname);
      cfg.cs_nickname = NULL;
    }

  cfg.cs_nickname = g_strdup(string);

  return TRUE;
}

gboolean gj_config_set_cs_resource(gchar *string) 
{
  if(string == NULL) 
    {
      g_warning("gj_config_set_cs_resource: resource was NULL");
      return FALSE;
    }

  if(cfg.cs_resource != NULL)
    {
      g_free(cfg.cs_resource);
      cfg.cs_resource = NULL;
    }

  cfg.cs_resource = g_strdup(string);

  return TRUE;
}

gboolean gj_config_set_cs_priority(gint8 integer) 
{
  cfg.cs_priority = integer;
  return TRUE;
}


gboolean gj_config_set_cs_save_password(gboolean bool) 
{
  if(bool == TRUE)
    cfg.cs_save_password = TRUE;
  else
    cfg.cs_save_password = FALSE;

  return TRUE;
}

gboolean gj_config_set_cs_use_plain_text_password(gboolean bool) 
{
  if(bool == TRUE)
    cfg.cs_use_plain_text_password = TRUE;
  else
    cfg.cs_use_plain_text_password = FALSE;

  return TRUE;
}

gboolean gj_config_set_cs_use_ssl(gboolean bool) 
{
  if(bool == TRUE)
    cfg.cs_use_ssl = TRUE;
  else
    cfg.cs_use_ssl = FALSE;

  return TRUE;
}

gboolean gj_config_set_cs_auto_connect_on_startup(gboolean bool) 
{
  if(bool == TRUE)
    cfg.cs_auto_connect_on_startup = TRUE;
  else
    cfg.cs_auto_connect_on_startup = FALSE;

  return TRUE;
}

gboolean gj_config_set_cs_auto_reconnect_on_disconnect(gboolean bool) 
{
  if(bool == TRUE)
    cfg.cs_auto_reconnect_on_disconnect = TRUE;
  else
    cfg.cs_auto_reconnect_on_disconnect = FALSE;

  return TRUE;
}

gboolean gj_config_set_mpf_hide_offline_contacts(gboolean bool)
{
  if(bool == TRUE)
    cfg.mpf_hide_offline_contacts = TRUE;
  else
    cfg.mpf_hide_offline_contacts = FALSE;

  return TRUE;
}

gboolean gj_config_set_mpf_hide_offline_group(gboolean bool)
{
  if(bool == TRUE)
    cfg.mpf_hide_offline_group = TRUE;
  else
    cfg.mpf_hide_offline_group = FALSE;

  return TRUE;
}

gboolean gj_config_set_mpf_hide_agents(gboolean bool)
{
  if(bool == TRUE)
    cfg.mpf_hide_agents = TRUE;
  else
    cfg.mpf_hide_agents = FALSE;

  return TRUE;
}

gboolean gj_config_set_mpf_hide_contact_presence_details(gboolean bool)
{
  if(bool == TRUE)
    cfg.mpf_hide_contact_presence_details = TRUE;
  else
    cfg.mpf_hide_contact_presence_details = FALSE;

  return TRUE;
}

gboolean gj_config_set_mpf_double_click_for_chat(gboolean bool)
{
  if(bool == TRUE)
    cfg.mpf_double_click_for_chat = TRUE;
  else
    cfg.mpf_double_click_for_chat = FALSE;

  return TRUE;
}

gboolean gj_config_set_mpf_close_previous_message_window_on_reply(gboolean bool)
{
  if(bool == TRUE)
    cfg.mpf_close_previous_message_window_on_reply = TRUE;
  else
    cfg.mpf_close_previous_message_window_on_reply = FALSE;

  return TRUE;
}

gboolean gj_config_set_mpf_raise_window_on_new_message(gboolean bool)
{
  if(bool == TRUE)
    cfg.mpf_raise_window_on_new_message = TRUE;
  else
    cfg.mpf_raise_window_on_new_message  = FALSE;

  return TRUE;
}

gboolean gj_config_set_mpf_hide_time(gboolean bool)
{
  if(bool == TRUE)
    cfg.mpf_hide_time = TRUE;
  else
    cfg.mpf_hide_time  = FALSE;

  return TRUE;
} 

gboolean gj_config_set_mpf_reverse_text_direction(gboolean bool)
{
  if(bool == TRUE)
    cfg.mpf_reverse_text_direction = TRUE;
  else
    cfg.mpf_reverse_text_direction  = FALSE;

  return TRUE;
} 

gboolean gj_config_set_mpf_use_spell_checker(gboolean bool)
{
  if(bool == TRUE)
    cfg.mpf_use_spell_checker = TRUE;
  else
    cfg.mpf_use_spell_checker = FALSE;

  return TRUE;
}

gboolean gj_config_set_mpf_auto_away_after(guint integer)
{
  if(integer < 1 || integer > 1000)
    {
      g_warning("gj_config_set_mpf_auto_away_after: value:%d was out of range (1 to 1000)", integer);
      return FALSE;
    }

  cfg.mpf_auto_away_after = integer;

  return TRUE;
}

gboolean gj_config_set_mpf_auto_xa_after(guint integer)
{
  if(integer < 1 || integer > 1000)
    {
      g_warning("gj_config_set_mpf_auto_xa_after: value:%d was out of range (1 to 1000)", integer);
      return FALSE;
    }

  cfg.mpf_auto_xa_after = integer;

  return TRUE;
}

gboolean gj_config_set_mpf_auto_away_status(gchar *string)
{
  if(string == NULL) 
    {
      g_warning("gj_config_set_mpf_auto_away_status: status was NULL");
      return FALSE;
    }

  if(cfg.mpf_auto_away_status != NULL)
    {
      g_free(cfg.mpf_auto_away_status);
      cfg.mpf_auto_away_status = NULL;
    }

  cfg.mpf_auto_away_status = g_strdup(string);

  return TRUE;
}

gboolean gj_config_set_mpf_auto_xa_status(gchar *string)
{
  if(string == NULL) 
    {
      g_warning("gj_config_set_mpf_auto_xa_status: status was NULL");
      return FALSE;
    }

  if(cfg.mpf_auto_xa_status != NULL)
    {
      g_free(cfg.mpf_auto_xa_status);
      cfg.mpf_auto_xa_status = NULL;
    }

  cfg.mpf_auto_xa_status = g_strdup(string);

  return TRUE;
}

gboolean gj_config_set_mpf_sound_on_user_online(gboolean bool)
{
  if(bool == TRUE)
    cfg.mpf_sound_on_user_online = TRUE;
  else
    cfg.mpf_sound_on_user_online  = FALSE;

  return TRUE;
} 

gboolean gj_config_set_mpf_sound_on_user_offline(gboolean bool)
{
  if(bool == TRUE)
    cfg.mpf_sound_on_user_offline = TRUE;
  else
    cfg.mpf_sound_on_user_offline  = FALSE;

  return TRUE;
} 

gboolean gj_config_set_mpf_sound_on_receive_message(gboolean bool)
{
  if(bool == TRUE)
    cfg.mpf_sound_on_receive_message = TRUE;
  else
    cfg.mpf_sound_on_receive_message  = FALSE;

  return TRUE;
} 

gboolean gj_config_set_mpf_sound_on_receive_chat(gboolean bool)
{
  if(bool == TRUE)
    cfg.mpf_sound_on_receive_chat = TRUE;
  else
    cfg.mpf_sound_on_receive_chat  = FALSE;

  return TRUE;
} 

gboolean gj_config_set_mpf_sound_on_receive_url(gboolean bool)
{
  if(bool == TRUE)
    cfg.mpf_sound_on_receive_url = TRUE;
  else
    cfg.mpf_sound_on_receive_url  = FALSE;

  return TRUE;
} 

gboolean gj_config_set_mpf_sound_on_receive_error(gboolean bool)
{
  if(bool == TRUE)
    cfg.mpf_sound_on_receive_error = TRUE;
  else
    cfg.mpf_sound_on_receive_error  = FALSE;

  return TRUE;
} 

gboolean gj_config_set_mpf_sound_file_on_user_online(gchar *string)
{
  if(string == NULL) 
    {
      g_warning("gj_config_set_mpf_sound_file_on_user_online: location was NULL");
      return FALSE;
    }

  if(cfg.mpf_sound_file_on_user_online != NULL)
    {
      g_free(cfg.mpf_sound_file_on_user_online);
      cfg.mpf_sound_file_on_user_online = NULL;
    }

  cfg.mpf_sound_file_on_user_online = g_strdup(string);

  return TRUE;
}

gboolean gj_config_set_mpf_sound_file_on_user_offline(gchar *string)
{
  if(string == NULL) 
    {
      g_warning("gj_config_set_mpf_sound_file_on_user_offline: location was NULL");
      return FALSE;
    }

  if(cfg.mpf_sound_file_on_user_offline != NULL)
    {
      g_free(cfg.mpf_sound_file_on_user_offline);
      cfg.mpf_sound_file_on_user_offline = NULL;
    }

  cfg.mpf_sound_file_on_user_offline = g_strdup(string);

  return TRUE;
}

gboolean gj_config_set_mpf_sound_file_on_receive_message(gchar *string)
{
  if(string == NULL) 
    {
      g_warning("gj_config_set_mpf_sound_file_on_receive_message: location was NULL");
      return FALSE;
    }

  if(cfg.mpf_sound_file_on_receive_message != NULL)
    {
      g_free(cfg.mpf_sound_file_on_receive_message);
      cfg.mpf_sound_file_on_receive_message = NULL;
    }

  cfg.mpf_sound_file_on_receive_message = g_strdup(string);

  return TRUE;
}

gboolean gj_config_set_mpf_sound_file_on_receive_chat(gchar *string)
{
  if(string == NULL) 
    {
      g_warning("gj_config_set_mpf_sound_file_on_receive_chat: location was NULL");
      return FALSE;
    }

  if(cfg.mpf_sound_file_on_receive_chat != NULL)
    {
      g_free(cfg.mpf_sound_file_on_receive_chat);
      cfg.mpf_sound_file_on_receive_chat = NULL;
    }

  cfg.mpf_sound_file_on_receive_chat = g_strdup(string);

  return TRUE;
}

gboolean gj_config_set_mpf_sound_file_on_receive_url(gchar *string)
{
  if(string == NULL) 
    {
      g_warning("gj_config_set_mpf_sound_file_on_receive_url: location was NULL");
      return FALSE;
    }

  if(cfg.mpf_sound_file_on_receive_url != NULL)
    {
      g_free(cfg.mpf_sound_file_on_receive_url);
      cfg.mpf_sound_file_on_receive_url = NULL;
    }

  cfg.mpf_sound_file_on_receive_url = g_strdup(string);

  return TRUE;
}

gboolean gj_config_set_mpf_sound_file_on_receive_error(gchar *string)
{
  if(string == NULL) 
    {
      g_warning("gj_config_set_mpf_sound_file_on_receive_error: location was NULL");
      return FALSE;
    }

  if(cfg.mpf_sound_file_on_receive_error != NULL)
    {
      g_free(cfg.mpf_sound_file_on_receive_error);
      cfg.mpf_sound_file_on_receive_error = NULL;
    }

  cfg.mpf_sound_file_on_receive_error = g_strdup(string);

  return TRUE;
}

gboolean gj_config_set_mpf_use_docklet(gboolean bool)
{
  if(bool == TRUE)
    cfg.mpf_use_docklet = TRUE;
  else
    cfg.mpf_use_docklet  = FALSE;

  return TRUE;
} 

gboolean gj_config_set_mpf_log_location(gchar *string)
{
  if(string == NULL) 
    {
      g_warning("gj_config_set_mpf_log_location: location was NULL");
      return FALSE;
    }

  if(cfg.mpf_log_location != NULL)
    {
      g_free(cfg.mpf_log_location);
      cfg.mpf_log_location = NULL;
    }

  cfg.mpf_log_location = g_strdup(string);

  return TRUE;
}

gboolean gj_config_set_mpf_history_location(gchar *string)
{
  if(string == NULL) 
    {
      g_warning("gj_config_set_mpf_history_location: location was NULL");
      return FALSE;
    }

  if(cfg.mpf_history_location != NULL)
    {
      g_free(cfg.mpf_history_location);
      cfg.mpf_history_location = NULL;
    }

  cfg.mpf_history_location = g_strdup(string);

  return TRUE;
}

gboolean gj_config_set_mpf_use_history(gboolean bool)
{
  if(bool == TRUE)
    cfg.mpf_use_history = TRUE;
  else
    cfg.mpf_use_history  = FALSE;

  return TRUE;
} 


#ifdef __cplusplus
}
#endif
