/* 
 * $RCSfile: gj_gatherdata.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/07/17 18:43:49 $
 * 
 * $Revision: 1.9 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <glib.h>

#include <libxml/tree.h>
#include <libxml/parser.h>

#include "gj_gatherdata.h"
#include "gj_parser.h"
#include "gj_presence.h"
#include "gj_message.h"
#include "gj_iq_requests.h"

/* gatherdata item */
void gj_gatherdata_destroy(gatherdata gd);

/* gatherdata list */
gboolean gj_gatherdata_list_add(gatherdata gd);
gboolean gj_gatherdata_list_del(gatherdata gd);

/* gatherdata field item */
void gj_gatherdata_field_destroy(gatherdata_field gdf);

/* gatherdata field list */
gboolean gj_gatherdata_field_list_add(gatherdata gd, gatherdata_field gdf);
gboolean gj_gatherdata_field_list_del(gatherdata gd, gatherdata_field gdf);

gboolean gj_gatherdata_field_list_free(gatherdata gd);

/* gatherdata field option item */
void gj_gatherdata_field_option_destroy(gatherdata_field_option gdfo);

/* gatherdata field option list */
gboolean gj_gatherdata_field_option_list_add(gatherdata_field gdf, gatherdata_field_option gdfo);
gboolean gj_gatherdata_field_option_list_del(gatherdata_field gdf, gatherdata_field_option gdfo);

gboolean gj_gatherdata_field_option_list_free(gatherdata_field gdf);

/* parsing functions */
void gj_gatherdata_cb(guint eventID);

/* parsing functions */
/* GList *gatherdataListParse(xmlNodePtr parent); */
GList *gj_gatherdata_parse(xmlNodePtr parent);

/* call backs */
/* gboolean gatherdataSetResponseProc(gatherdataResponseProc proc); */
/* gatherdataResponseProc gatherdataGetResponseProc(); */

struct t_gatherdata_field_option
{
  gchar *label;
  gchar *value;
};

struct t_gatherdata_field
{
  gatherdataFieldType type;

  gchar *label;
  gchar *var;
  gchar *value;

  gboolean required;

  GList *options;
};

struct t_gatherdata
{
  gchar *id;
  gchar *to;
  gchar *from;

  gatherdataFormType type; 
 
  gchar *instructions;
  GList *fields;

  gatherdataCallback cb; 

  gchar *related_id;
};

GList *gatherdata_list = NULL;
gatherdataRequestCallback gatherdata_cb = NULL;

gboolean gj_gatherdata_init()
{
  if(gatherdata_list != NULL)
    gj_gatherdata_term();

  /* create mem  - nothing to do, created on g_list_append */
  return TRUE;
}

gboolean gj_gatherdata_term()
{
  gint index = 0;

  /* list */
  if(gatherdata_list == NULL)
    return TRUE;

  /* free all items on the list */
  for(index=0;index<g_list_length(gatherdata_list);index++)
    {
      gatherdata gd = NULL;
      
      if((gd = (gatherdata) g_list_nth_data(gatherdata_list,index)) != NULL)
	gj_gatherdata_free(gd);
    }

  g_list_free(gatherdata_list);
  gatherdata_list = NULL;

  return TRUE;
}

gboolean gj_gatherdata_list_add(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_list_add: gd was NULL");
      return FALSE;
    }

  gatherdata_list = g_list_append(gatherdata_list, gd);

  return TRUE;
}

gboolean gj_gatherdata_list_del(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_list_del: gd was NULL");
      return FALSE;
    }

  gatherdata_list = g_list_remove(gatherdata_list, gd);
  return TRUE;
}

GList *gatherdata_listGet()
{
  return gatherdata_list;
}

/* 
 * item functions
 */
gatherdata gj_gatherdata_new()
{
  gatherdata gd;

  if((gd = g_new(struct t_gatherdata,1)))
    {
      gd->id = NULL;
      gd->to = NULL;
      gd->from = NULL;
      
      gd->instructions = NULL;
      gd->fields = NULL;
      
      gd->cb = NULL;

      gd->related_id = NULL;

      gj_gatherdata_list_add(gd);
    }
  
  return gd;  
}

void gj_gatherdata_free(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_free: gd was NULL");
      return;
    }

  /* free mem */
  gj_gatherdata_list_del(gd);  

  /* destroy members */
  gj_gatherdata_destroy(gd);

  /* free structure */
  g_free(gd);
}

void gj_gatherdata_destroy(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_destroy: gd was NULL");
      return;
    }
  
  if(gd->id != NULL)
    {
      g_free(gd->id);
      gd->id = NULL;
    }

  if(gd->to != NULL)
    {
      g_free(gd->to);
      gd->to = NULL;
    }

  if(gd->from != NULL)
    {
      g_free(gd->from);
      gd->from = NULL;
    }

  if(gd->instructions != NULL)
    {
      g_free(gd->instructions);
      gd->instructions = NULL;
    }

  gj_gatherdata_field_list_free(gd);
  gd->fields = NULL;

  gd->cb = NULL;

  if(gd->related_id != NULL)
    {
      g_free(gd->related_id);
      gd->related_id = NULL;
    }
}

/*
 * find 
 */
gatherdata gj_gatherdata_find_by_id(gchar *id)
{
  gint index = 0;

  if(id == NULL)
    {
      g_warning("gj_gatherdata_find_by_id: name was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(gatherdata_list);index++)
    {
      gatherdata gd  = g_list_nth_data(gatherdata_list,
				       index);

      if(gd == NULL)
	{
	  g_warning("gj_gatherdata_find_by_id: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(g_strcasecmp(gd->id, id) == 0)
	return gd;
    }

  return NULL;
}

gatherdata gj_gatherdata_find_by_to(gchar *to)
{
  gint index = 0;

  if(to == NULL)
    {
      g_warning("gj_gatherdata_find_by_to: name was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(gatherdata_list);index++)
    {
      gatherdata gd  = g_list_nth_data(gatherdata_list,
				       index);

      if(gd == NULL)
	{
	  g_warning("gj_gatherdata_find_by_to: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(g_strcasecmp(gd->to, to) == 0)
	return gd;
    }

  return NULL;
}

gatherdata gj_gatherdata_find_by_from(gchar *from)
{
  gint index = 0;

  if(from == NULL)
    {
      g_warning("gj_gatherdata_find_by_from: name was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(gatherdata_list);index++)
    {
      gatherdata gd  = g_list_nth_data(gatherdata_list,
				       index);

      if(gd == NULL)
	{
	  g_warning("gj_gatherdata_find_by_from: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(g_strcasecmp(gd->from, from) == 0)
	return gd;
    }

  return NULL;
}

gatherdata gj_gatherdata_find_by_related_id(gchar *id)
{
  gint index = 0;

  if(id == NULL)
    {
      g_warning("gj_gatherdata_find_by_related_id: name was NULL");
      return NULL;
    }

  for(index=0;index<g_list_length(gatherdata_list);index++)
    {
      gatherdata gd  = g_list_nth_data(gatherdata_list,
				       index);

      if(gd == NULL)
	{
	  g_warning("gj_gatherdata_find_by_related_id: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      if(g_strcasecmp(gd->related_id, id) == 0)
	return gd;
    }

  return NULL;
}

/*
 * member functions  
 */

/* sets */
gboolean gj_gatherdata_set_id(gatherdata gd, gchar *id)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_set_id: gd was NULL");
      return FALSE;
    }

  if(id == NULL)
    {
      g_warning("gj_gatherdata_set_id: id was NULL");
      return FALSE;
    }

  if(gd->id != NULL)
    g_free(gd->id);
  
  gd->id = g_strdup(id);

  return TRUE;
}

gboolean gj_gatherdata_set_to(gatherdata gd, gchar *to)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_set_to: gd was NULL");
      return FALSE;
    }

  if(to == NULL)
    {
      g_warning("gj_gatherdata_set_to: to was NULL");
      return FALSE;
    }

  if(gd->to != NULL)
    g_free(gd->to);
  
  gd->to = g_strdup(to);

  return TRUE;
}

gboolean gj_gatherdata_set_from(gatherdata gd, gchar *from)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_set_from: gd was NULL");
      return FALSE;
    }

  if(from == NULL)
    {
      g_warning("gj_gatherdata_set_from: from was NULL");
      return FALSE;
    }

  if(gd->from != NULL)
    g_free(gd->from);
  
  gd->from = g_strdup(from);

  return TRUE;
}

gboolean gj_gatherdata_set_type(gatherdata gd, gatherdataFormType type)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_set_type: gd was NULL");
      return FALSE;
    }
  
  if(type < gatherdataFormTypeForm || type > gatherdataFormTypeEnd)
    {
      g_warning("gj_gatherdata_set_type: type:%d was out of valid range", type);
      return FALSE;
    }

  gd->type = type;
  return TRUE;
}

gboolean gj_gatherdata_set_instructions(gatherdata gd, gchar *instructions)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_set_instructions: gd was NULL");
      return FALSE;
    }

  if(instructions == NULL)
    {
      g_warning("gj_gatherdata_set_instructions: id was NULL");
      return FALSE;
    }

  if(gd->instructions != NULL)
    g_free(gd->instructions);
  
  gd->instructions = g_strdup(instructions);

  return TRUE;
}

gboolean gj_gatherdata_set_fields(gatherdata gd, GList *fields)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_set_fields: gd was NULL");
      return FALSE;
    }
  
  if(fields == NULL)
    {
      g_warning("gj_gatherdata_set_fields: fields list was NULL");
      return FALSE;
    }

  gd->fields = fields;
  return TRUE;
}

gboolean gj_gatherdata_set_callback(gatherdata gd, gatherdataCallback cb)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_set_callback: gd was NULL");
      return FALSE;
    }
  
  if(cb == NULL)
    {
      g_warning("gj_gatherdata_set_fields: callback was NULL");
      return FALSE;
    }

  gd->cb = cb;
  return TRUE;
}

gboolean gj_gatherdata_set_related_id(gatherdata gd, gchar *id)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_set_related_id: gd was NULL");
      return FALSE;
    }

  if(id == NULL)
    {
      g_warning("gj_gatherdata_set_related_id: id was NULL");
      return FALSE;
    }

  if(gd->related_id != NULL)
    g_free(gd->related_id);
  
  gd->related_id = g_strdup(id);

  return TRUE;
}

/* gets */
gchar *gj_gatherdata_get_id(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_get_id: gd was NULL");
      return NULL;
    }

  return gd->id;
}

gchar *gj_gatherdata_get_to(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_get_to: gd was NULL");
      return NULL;
    }

  return gd->to;
}

gchar *gj_gatherdata_get_from(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_get_from: gd was NULL");
      return NULL;
    }

  return gd->from;
}

gatherdataFormType gj_gatherdata_get_type(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_get_type: gd was NULL");
      return gatherdataFormTypeEnd;
    }

  return gd->type;
}

gchar *gj_gatherdata_get_instructions(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_get_instructions: gd was NULL");
      return NULL;
    }

  return gd->instructions;
}

GList *gj_gatherdata_get_fields(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_get_fields: gd was NULL");
      return NULL;
    }

  return gd->fields;
}

gatherdataCallback gj_gatherdata_get_callback(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_get_callback: gd was NULL");
      return NULL;
    }

  return gd->cb;
}

gchar *gj_gatherdata_get_related_id(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_get_related_id: gd was NULL");
      return NULL;
    }

  return gd->related_id;
}


/* debugging */
void gj_gatherdata_print_all()
{
  gatherdata gd = NULL;
  gint index = 0;

  g_message("++++++++++++++++++++++ Gatherdata - Print All, Begin. +++++++++++++++++++++");

  for(index=0;index<g_list_length(gatherdata_list);index++)
    {
      gd = (gatherdata) g_list_nth_data(gatherdata_list,index);
      gj_gatherdata_print(gd);
    }

  g_message("++++++++++++++++++++++ Gatherdata - Print All, End.   +++++++++++++++++++++");
  
  return;  
}

void gj_gatherdata_print(gatherdata gd)
{
  if(gd == NULL)
    return;

  g_message("+++++++++++++++++++++++++++ Gatherdata +++++++++++++++++++++++++");
  g_message("++++");
  g_message("++++ id:'%s'",gd->id);
  g_message("++++ to:'%s'",gd->to);
  g_message("++++ from:'%s'",gd->from);
  g_message("++++");
  g_message("++++ instructions:'%s'",gd->instructions);
  g_message("++++");
  g_message("++++ related id:'%s'",gd->related_id);
  g_message("++++");
  g_message("+++++++++++++++++++++++++++ Gatherdata +++++++++++++++++++++++++");
}

/*
 *
 * GATHERDATA FIELD functions
 *
 */
gboolean gj_gatherdata_field_list_add(gatherdata gd, gatherdata_field gdf)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_field_list_add: gd was NULL");
      return FALSE;
    }

  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_list_add: gdf was NULL");
      return FALSE;
    }

  gd->fields = g_list_append(gd->fields, gd);

  return TRUE;
}

gboolean gj_gatherdata_field_list_del(gatherdata gd, gatherdata_field gdf)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_field_list_del: gd was NULL");
      return FALSE;
    }

  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_list_del: gdf was NULL");
      return FALSE;
    }

  gd->fields = g_list_remove(gd->fields, gdf);
  return TRUE;
}

gboolean gj_gatherdata_field_list_free(gatherdata gd)
{
  gint index = 0;

  if(gd == NULL)
    {
      g_warning("gj_gatherdata_field_list_free: gd was NULL");
      return FALSE;
    }
  
  if(gj_gatherdata_get_fields(gd) == NULL)
    return TRUE;

  for(index=0;index<g_list_length(gj_gatherdata_get_fields(gd));index++)
    {
      gatherdata_field gdf = g_list_nth_data(gj_gatherdata_get_fields(gd), index);
      
      if(gdf == NULL)
	{
	  g_warning("gj_gatherdata_field_list_free: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      gj_gatherdata_field_free(gd, gdf);
    }

  return TRUE;
}

GList *gj_gatherdata_field_list_get(gatherdata gd)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_field_list_get: gd was NULL");
      return NULL;
    }

  return gd->fields;
}

/* 
 * item functions
 */
gatherdata_field gj_gatherdata_field_new(gatherdata gd, gatherdataFieldType type)
{
  gatherdata_field gdf = NULL;

  if(gd == NULL)
    {
      g_warning("gj_gatherdata_field_new: gd was NULL");
      return NULL;
    }

  if(type < 1 || type > gatherdataFieldTypeEnd)
    {
      g_warning("gj_gatherdata_field_new: type:%d was out of bounds", type);
      return NULL;
    }

  if((gdf = g_new(struct t_gatherdata_field,1)))
    {
      gdf->type = type;

      gdf->label = NULL;
      gdf->var = NULL;
      gdf->value = NULL;
      
      gdf->required = FALSE;

      gdf->options = NULL;

      gj_gatherdata_field_list_add(gd, gdf);
    }
  
  return gdf;  
}

void gj_gatherdata_field_free(gatherdata gd, gatherdata_field gdf)
{
  if(gd == NULL)
    {
      g_warning("gj_gatherdata_field_free: gd was NULL");
      return;
    }  

  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_free: gdf was NULL");
      return;
    }

  /* free mem */
  gj_gatherdata_field_list_del(gd, gdf);  

  /* destroy members */
  gj_gatherdata_field_destroy(gdf);

  /* free structure */
  g_free(gdf);
}

void gj_gatherdata_field_destroy(gatherdata_field gdf)
{
  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_destroy: gdf was NULL");
      return;
    }

  gdf->type = gatherdataFieldTypeEnd;
  
  if(gdf->label != NULL)
    {
      g_free(gdf->label);
      gdf->label = NULL;
    }

  if(gdf->var != NULL)
    {
      g_free(gdf->var);
      gdf->var = NULL;
    }

  if(gdf->value != NULL)
    {
      g_free(gdf->value);
      gdf->value = NULL;
    }

  gdf->required = FALSE;

  gj_gatherdata_field_option_list_free(gdf);
  gdf->options = NULL;  
}

/*
 * member functions  
 */

/* sets */
gboolean gj_gatherdata_field_set_type(gatherdata_field gdf, gatherdataFieldType type)
{
  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_set_type: gdf was NULL");
      return FALSE;
    }

  if(type < 1 || type > gatherdataFieldTypeEnd)
    {
      g_warning("gj_gatherdata_field_set_type: type:%d was out of bounds", type);
      return FALSE;
    }

  gdf->type = type;

  return TRUE;
}

gboolean gj_gatherdata_field_set_label(gatherdata_field gdf, gchar *label)
{
  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_set_label: gdf was NULL");
      return FALSE;
    }

  if(label == NULL)
    {
      g_warning("gj_gatherdata_field_set_label: label was NULL");
      return FALSE;
    }

  if(gdf->label != NULL)
    g_free(gdf->label);
  
  gdf->label = g_strdup(label);

  return TRUE;
}

gboolean gj_gatherdata_field_set_var(gatherdata_field gdf, gchar *var)
{
  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_set_var: gdf was NULL");
      return FALSE;
    }

  if(var == NULL)
    {
      g_warning("gj_gatherdata_field_set_var: var was NULL");
      return FALSE;
    }

  if(gdf->var != NULL)
    g_free(gdf->var);
  
  gdf->var = g_strdup(var);

  return TRUE;
}

gboolean gj_gatherdata_field_set_value(gatherdata_field gdf, gchar *value)
{
  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_set_value: gdf was NULL");
      return FALSE;
    }

  if(value == NULL)
    {
      g_warning("gj_gatherdata_field_set_value: value was NULL");
      return FALSE;
    }

  if(gdf->value != NULL)
    g_free(gdf->value);
  
  gdf->value = g_strdup(value);

  return TRUE;
}

gboolean gj_gatherdata_field_set_required(gatherdata_field gdf, gboolean required)
{
  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_set_required: gdf was NULL");
      return FALSE;
    }

  gdf->required = required;

  return TRUE;
}

gboolean gj_gatherdata_field_set_options(gatherdata_field gdf, GList *options)
{
  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_set_options: gdf was NULL");
      return FALSE;
    }

  /* warning ... what happens to previous list? */
  gdf->options = options;

  return TRUE;
}

/* gets */
gatherdataFieldType gj_gatherdata_field_get_type(gatherdata_field gdf)
{
  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_get_type: gdf was NULL");
      return gatherdataFieldTypeEnd;
    }

  return gdf->type;
}


gchar *gj_gatherdata_field_get_label(gatherdata_field gdf)
{
  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_get_label: gdf was NULL");
      return NULL;
    }

  return gdf->label;
}


gchar *gj_gatherdata_field_get_var(gatherdata_field gdf)
{
  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_get_var: gdf was NULL");
      return NULL;
    }

  return gdf->var;
}


gchar *gj_gatherdata_field_get_value(gatherdata_field gdf)
{
  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_get_value: gdf was NULL");
      return NULL;
    }

  return gdf->value;
}


gboolean gj_gatherdata_field_get_required(gatherdata_field gdf)
{
  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_get_required: gdf was NULL");
      return FALSE;
    }

  return gdf->required;
}


GList *gj_gatherdata_field_get_options(gatherdata_field gdf)
{
  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_get_options: gdf was NULL");
      return NULL;
    }

  return gdf->options;
}

/*
 *
 * GATHERDATA FIELD OPTION functions
 *
 */
gboolean gj_gatherdata_field_option_list_add(gatherdata_field gdf, gatherdata_field_option gdfo)
{
  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_option_list_add: gdf was NULL");
      return FALSE;
    }

  if(gdfo == NULL)
    {
      g_warning("gj_gatherdata_field_option_list_add: gdfo was NULL");
      return FALSE;
    }

  gdf->options = g_list_append(gdf->options, gdfo);

  return TRUE;
}

gboolean gj_gatherdata_field_option_list_del(gatherdata_field gdf, gatherdata_field_option gdfo)
{
  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_option_list_del: gdf was NULL");
      return FALSE;
    }

  if(gdfo == NULL)
    {
      g_warning("gj_gatherdata_field_option_list_del: gdfo was NULL");
      return FALSE;
    }

  gdf->options = g_list_remove(gdf->options, gdfo);
  return TRUE;
}

gboolean gj_gatherdata_field_option_list_free(gatherdata_field gdf)
{
  gint index = 0;

  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_option_list_free: gdf was NULL");
      return FALSE;
    }
  
  if(gj_gatherdata_field_get_options(gdf) == NULL)
    return TRUE;

  for(index=0;index<g_list_length(gj_gatherdata_field_get_options(gdf));index++)
    {
      gatherdata_field_option gdfo = g_list_nth_data(gj_gatherdata_field_get_options(gdf), index);
      
      if(gdfo == NULL)
	{
	  g_warning("gj_gatherdata_field_option_list_free: getting nth item(%d) in list was NULL",index);
	  continue;
	}

      gj_gatherdata_field_option_free(gdf, gdfo);
    }

  return TRUE;
}

GList *gj_gatherdata_field_option_list_get(gatherdata_field gdf)
{
  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_option_list_get: gd was NULL");
      return NULL;
    }

  return gdf->options;
}

/* 
 * item functions
 */
gatherdata_field_option gj_gatherdata_field_option_new(gatherdata_field gdf, gchar *label)
{
  gatherdata_field_option gdfo = NULL;

  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_option_new: gdf was NULL");
      return NULL;
    }

  if(label == NULL)
    {
      g_warning("gj_gatherdata_field_option_new: label was NULL");
      return NULL;
    }

  if((gdfo = g_new(struct t_gatherdata_field_option,1)))
    {
      gdf->label = g_strdup(label);
      gdf->value = NULL;

      gj_gatherdata_field_option_list_add(gdf, gdfo);
    }
  
  return gdfo;  
}

void gj_gatherdata_field_option_free(gatherdata_field gdf, gatherdata_field_option gdfo)
{
  if(gdf == NULL)
    {
      g_warning("gj_gatherdata_field_option_free: gdf was NULL");
      return;
    }  

  if(gdfo == NULL)
    {
      g_warning("gj_gatherdata_field_option_free: gdfo was NULL");
      return;
    }

  /* free mem */
  gj_gatherdata_field_option_list_del(gdf, gdfo);  

  /* destroy members */
  gj_gatherdata_field_option_destroy(gdfo);

  /* free structure */
  g_free(gdfo);
}

void gj_gatherdata_field_option_destroy(gatherdata_field_option gdfo)
{
  if(gdfo == NULL)
    {
      g_warning("gj_gatherdata_field_option_destroy: gdfo was NULL");
      return;
    }

  if(gdfo->label != NULL)
    {
      g_free(gdfo->label);
      gdfo->label = NULL;
    }

  if(gdfo->value != NULL)
    {
      g_free(gdfo->value);
      gdfo->value = NULL;
    }
}

/*
 * member functions  
 */

/* sets */
gboolean gj_gatherdata_field_option_set_label(gatherdata_field_option gdfo, gchar *label)
{
  if(gdfo == NULL)
    {
      g_warning("gj_gatherdata_field_option_set_label: gdfo was NULL");
      return FALSE;
    }

  if(label == NULL)
    {
      g_warning("gj_gatherdata_field_option_set_label: label was NULL");
      return FALSE;
    }

  if(gdfo->label != NULL)
    g_free(gdfo->label);
  
  gdfo->label = g_strdup(label);
  return TRUE;
}

gboolean gj_gatherdata_field_option_set_value(gatherdata_field_option gdfo, gchar *value)
{
  if(gdfo == NULL)
    {
      g_warning("gj_gatherdata_field_option_set_value: gdfo was NULL");
      return FALSE;
    }

  if(value == NULL)
    {
      g_warning("gj_gatherdata_field_option_set_value: value was NULL");
      return FALSE;
    }

  if(gdfo->value != NULL)
    g_free(gdfo->value);
  
  gdfo->value = g_strdup(value);
  return TRUE;
}

/* gets */
gchar *gj_gatherdata_field_option_get_label(gatherdata_field_option gdfo)
{
  if(gdfo == NULL)
    {
      g_warning("gj_gatherdata_field_option_get_label: gdfo was NULL");
      return NULL;
    }

  return gdfo->label;
}

gchar *gj_gatherdata_field_option_get_value(gatherdata_field_option gdfo)
{
  if(gdfo == NULL)
    {
      g_warning("gj_gatherdata_field_option_get_value: gdfo was NULL");
      return NULL;
    }

  return gdfo->value;
}





/*
 *
 * SERVICES
 *
 */
gboolean gj_gatherdata_request(gchar *server, gatherdataRequestCallback cb)
{
  if(server == NULL)
    {
      g_warning("gj_gatherdata_request: server was NULL");
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("gj_gatherdata_request: cb was NULL");
      return FALSE;
    }

  gatherdata_cb = cb;

  /* do request */
  gj_iq_request_register(server, gj_gatherdata_cb);

  return TRUE;
}

void gj_gatherdata_cb(guint eventID)
{
  infoquery iq = NULL;

  xmlChar *type = NULL;
  xmlNodePtr node = NULL;

  GList *list = NULL;
  
  gint error_code = 0;
  gchar *error_reason = NULL;

  if(gatherdata_cb == NULL)
    {
      g_warning("gj_gatherdata_cb: callback was NULL");
      return;
    }

  if((iq = gj_iq_find_by_event_id(eventID)) == NULL)
    {
      g_warning("gj_gatherdata_cb: could not get infoquery event id:%d", eventID);
      return;
    }

  node = gj_iq_get_xml(iq);
  type = gj_parser_find_by_attr_and_ret(node, (guchar*)"type");

  if(type != NULL && xmlStrcmp(type, (guchar*)"error") == 0)
    {
      xmlChar *errorCode = NULL;
      xmlChar *errorMessage = NULL;

      if((node = gj_parser_find_by_node(node, (guchar*)"error")) == NULL)
	{
	  g_warning("gj_gatherdata_cb: failed to find node 'error'");
	  gj_iq_free_by_event_id(eventID);
	  return;
	}

      if((errorCode = gj_parser_find_by_attr_and_ret(node, (guchar*)"code")) != NULL)
	error_code = atoi((gchar*)errorCode);

      if((errorMessage = gj_parser_find_by_node_and_ret(node, (guchar*)"error")) != NULL) 
	error_reason = g_strdup((gchar*)errorMessage);
    }

  if(gatherdata_list != NULL)
    {
      /* free list */

    }
  
  /* parse list */
  if((list = gj_gatherdata_parse(node)) == NULL)
    {
      g_warning("gj_gatherdata_cb: did not complete the parse from xml -> list");
      return;
    }

  /* set */
  gatherdata_list = list;

  /* return list */
  (gatherdata_cb)(gatherdata_list, error_code, error_reason);

  /* clean up */
  if(error_reason != NULL) g_free(error_reason);
  gj_iq_free_by_event_id(eventID);
}

GList *gj_gatherdata_parse(xmlNodePtr parent)
{
  if(parent == NULL)
    {
      g_warning("gj_gatherdata_parse: failed to get gatherdata because query (xml) was NULL");
      return NULL;
    }

  return NULL;
}

#ifdef __cplusplus
}
#endif
