/*
 * $RCSfile: gj_gtk_about.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/08/19 22:25:03 $
 *
 * $Revision: 1.13 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* gui */
#include <glade/glade.h>
#include <gtk/gtk.h>

/* local */
#include "common.h"

#include "gj_main.h"

#include "gj_gtk_about.h"
#include "gj_gtk_support.h"

/* local functions */
gboolean gj_gtk_about_setup(const gchar *window_title,
			    const gchar *title, 
			    const gchar *semititle, 
			    const gchar *version, 
			    const gchar *build, 
			    const gchar *copyright, 
			    const gchar *license,
			    const gchar *sn);

/* credits dialog */
gboolean gj_gtk_about_credits();

GtkWidget *gj_gtk_about_credits_create_label();

void gj_gtk_about_credits_update_authors(GtkWidget *label);
void gj_gtk_about_credits_update_documenters(GtkWidget *label);
void gj_gtk_about_credits_update_translators(GtkWidget *label);

/* event callbacks */
void on_about_response(GtkDialog *dialog, gint response, gpointer user_data);

struct 
{
  /* search */
  GtkWidget *window;

  GtkWidget *label_title;
  GtkWidget *label_semititle;
  GtkWidget *label_version;
  GtkWidget *label_build;
  GtkWidget *label_copyright;
  GtkWidget *label_license;
  GtkWidget *label_sn;

  GtkWidget *image_icon;

  GtkWidget *button_credits;
  GtkWidget *button_close;

  GtkWidget *credits;

  GSList *authors;
  GSList *documenters;
  GSList *translators;

} ctlAbout;

/* responses */
enum
  {
    ABOUT_RESPONSE_CREDITS = 1
  };


/*
 * external functions...
 */
gboolean gj_gtk_about_load(const gchar *window_title,
			   const gchar *title, 
			   const gchar *semititle, 
			   const gchar *version, 
			   const gchar *build, 
			   const gchar *copyright, 
			   const gchar *license,
			   const gchar *sn,
			   const gchar **authors,
			   const gchar **documenters,
			   const gchar **translators)
{
  GladeXML *xml = NULL;

  GValueArray *authors_array = NULL;
  GValueArray *documenters_array = NULL;
  GValueArray *translators_array = NULL;

  GValueArray *value_array;
  GSList *list;

  gint i = 0;

  if(ctlAbout.window != NULL)
    {
      gtk_window_present(GTK_WINDOW(ctlAbout.window));
      return FALSE;
    }
    
  /* check vars */
  if(window_title == NULL)
    {
      g_warning("%s: window_title was NULL", __FUNCTION__);
      return FALSE;
    }

  if(title == NULL)
    {
      g_warning("%s: title was NULL", __FUNCTION__);
      return FALSE;
    }

  if(semititle == NULL)
    {
      g_warning("%s: semititle was NULL", __FUNCTION__);
      return FALSE;
    }

  if(version == NULL)
    {
      g_warning("%s: version was NULL", __FUNCTION__);
      return FALSE;
    }

  if(build == NULL)
    {
      g_warning("%s: build was NULL", __FUNCTION__);
      return FALSE;
    }

  if(copyright == NULL)
    {
      g_warning("%s: copyright was NULL", __FUNCTION__);
      return FALSE;
    }

  if(license == NULL)
    {
      g_warning("%s: license was NULL", __FUNCTION__);
      return FALSE;
    }

  /*     if(sn == NULL) */
  /*     { */
  /* 	g_warning("gj_gtk_about_load: serial number was NULL"); */
  /* 	return FALSE; */
  /*     } */
    

  /* authors */
  g_message("%s: setting authors...", __FUNCTION__);
  authors_array = g_value_array_new (0);

  for (i = 0; authors[i] != NULL; i++) {
    GValue value = {0, };
 	
    g_value_init (&value, G_TYPE_STRING);
    g_value_set_static_string (&value, authors[i]);
    authors_array = g_value_array_append (authors_array, &value);
  }

  /* documenters */
  if(documenters != NULL) 
    {
      g_message("%s: setting documenters...", __FUNCTION__);
      documenters_array = g_value_array_new (0);
	
      for (i = 0; documenters[i] != NULL; i++) {
	GValue value = {0, };
	    
	g_value_init (&value, G_TYPE_STRING);
	g_value_set_static_string (&value, documenters[i]);
	documenters_array = g_value_array_append (documenters_array, &value);
      }
	
    }

  /* translators */
  if(translators != NULL) 
    {
      g_message("%s: setting translators...", __FUNCTION__);
      translators_array = g_value_array_new (0);
	
      for (i = 0; translators[i] != NULL; i++) {
	GValue value = {0, };
	    
	g_value_init (&value, G_TYPE_STRING);
	g_value_set_static_string (&value, translators[i]);
	translators_array = g_value_array_append (translators_array, &value);
      }
	
    }

  /* authors */
  list = ctlAbout.authors;
  if(list != NULL)
    {    
      g_slist_foreach(list, (GFunc) g_free, NULL);
      g_slist_free(list);
    }
    
  list = NULL;
  /*     value_array = g_value_get_boxed(authors_array); */
  value_array = authors_array;

  if(value_array != NULL)
    {
      for (i = 0; i < value_array->n_values; i++)
	list = g_slist_prepend (list, g_value_dup_string (&value_array->values[i]));
	
      list = g_slist_reverse(list);
      ctlAbout.authors = list;
    }

  /* documenters */
  list = ctlAbout.documenters;
  if(list != NULL)
    {
      g_slist_foreach(list, (GFunc) g_free, NULL);
      g_slist_free(list);
    }
                                                                                                                   
  list = NULL;
  /*     value_array = g_value_get_boxed(documenters_array); */
  value_array = documenters_array;
                                                                                                                   
  if(value_array != NULL)
    {
      for (i = 0; i < value_array->n_values; i++)
	list = g_slist_prepend (list, g_value_dup_string (&value_array->values[i]));
                                                                                                                   
      list = g_slist_reverse(list);
      ctlAbout.documenters = list;
    }

  /* translators */
  list = ctlAbout.translators;
  if(list != NULL)
    {    
      g_slist_foreach(list, (GFunc) g_free, NULL);
      g_slist_free(list);
    }
    
  list = NULL;
  /*     value_array = g_value_get_boxed(documenters_array); */
  value_array = translators_array;
    
  if(value_array != NULL)
    {
      for (i = 0; i < value_array->n_values; i++)
	list = g_slist_prepend (list, g_value_dup_string (&value_array->values[i]));
	
      list = g_slist_reverse(list);
      ctlAbout.translators = list;
    }

  /* load dialog */
  if((xml = glade_xml_new(gnome_jabber_get_glade_file(),
			  "about",
			  NULL)) == NULL)
    {
      g_warning("%s: couldn't open glade file at root 'about'", __FUNCTION__);
      return FALSE;
    }
  
  /* controls */
  ctlAbout.window = glade_xml_get_widget(xml, "about");

  ctlAbout.label_title = glade_xml_get_widget(xml, "label_about_title");
  ctlAbout.label_semititle = glade_xml_get_widget(xml, "label_about_semititle");
  ctlAbout.label_version = glade_xml_get_widget(xml, "label_about_version");
  ctlAbout.label_build = glade_xml_get_widget(xml, "label_about_build");
  ctlAbout.label_copyright = glade_xml_get_widget(xml, "label_about_copyright");
  ctlAbout.label_license = glade_xml_get_widget(xml, "label_about_license");
  ctlAbout.label_sn = glade_xml_get_widget(xml, "label_about_sn");

  ctlAbout.image_icon = glade_xml_get_widget(xml, "image_about_icon");

  ctlAbout.button_credits = glade_xml_get_widget(xml, "button_about_credits");
  ctlAbout.button_close = glade_xml_get_widget(xml, "button_about_close");

  gdk_window_set_decorations (GDK_WINDOW(ctlAbout.window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE);

  /* signals */
#ifndef WIN32
  glade_xml_signal_autoconnect(xml);
#else
  g_signal_connect ((gpointer) ctlAbout.window, "response",
		    G_CALLBACK (on_about_response),
		    NULL);
#endif

  /* check window is created ok */
  if(ctlAbout.window == NULL)
    {
      g_warning("%s: failed to create message box, window was NULL", __FUNCTION__);
      return FALSE;
    }

  g_message("%s: using window title:'%s'", __FUNCTION__, window_title);
  g_message("%s: using title:'%s'", __FUNCTION__, title);
  g_message("%s: using semititle:'%s'", __FUNCTION__, semititle);
  g_message("%s: using version:'%s'", __FUNCTION__, version);
  g_message("%s: using build:'%s'", __FUNCTION__, build);
  g_message("%s: using copyright:'%s'", __FUNCTION__, copyright);
  g_message("%s: using license:'%s'", __FUNCTION__, license);
  g_message("%s: using serial number:'%s'", __FUNCTION__, sn);

  gj_gtk_about_setup(window_title,
		     title,
		     semititle,
		     version,
		     build,
		     copyright,
		     license,
		     sn);

  /* clean up */
  if(xml != NULL)
    {
      g_object_unref(G_OBJECT(xml));
      xml = NULL;
    }

  if(authors_array != NULL) g_value_array_free(authors_array);
  if(documenters_array != NULL) g_value_array_free(documenters_array);
  if(translators_array != NULL) g_value_array_free(translators_array);

  return TRUE;
}

gboolean gj_gtk_about_unload()
{
  gtk_widget_destroy(ctlAbout.window);
  ctlAbout.window = NULL;
  ctlAbout.credits = NULL;
    
  if(ctlAbout.authors != NULL)
    {    
      g_slist_foreach(ctlAbout.authors, (GFunc) g_free, NULL);
      g_slist_free(ctlAbout.authors);
    }

  if(ctlAbout.documenters != NULL)
    {    
      g_slist_foreach(ctlAbout.documenters, (GFunc) g_free, NULL);
      g_slist_free(ctlAbout.documenters);
    }

  if(ctlAbout.translators != NULL)
    {
      g_slist_foreach(ctlAbout.translators, (GFunc) g_free, NULL);
      g_slist_free(ctlAbout.translators);
    }


  ctlAbout.authors = NULL;
  ctlAbout.documenters = NULL;
  ctlAbout.translators = NULL;

  return TRUE;
}

gboolean gj_gtk_about_setup(const gchar *window_title,
			    const gchar *title, 
			    const gchar *semititle, 
			    const gchar *version, 
			    const gchar *build, 
			    const gchar *copyright, 
			    const gchar *license,
			    const gchar *sn)
{
  gchar *text = NULL;
  GdkPixbufAnimation *ani = NULL; 

#if defined G_OS_WIN32
  const char *os = "Windows";
#elif defined G_OS_UNIX
  const char *os = "Unix";
#elif defined G_OS_BEOS
  const char *os = "BeOS";
#else
  const char *os = "unknown operating system";
#endif

  if(window_title != NULL)
    gtk_window_set_title(GTK_WINDOW(ctlAbout.window), window_title);

  if(title != NULL)
    {
      text = g_strdup_printf("<span size=\"xx-large\" weight=\"heavy\" stretch=\"ultraexpanded\">%s</span>", 
			     title);
      gtk_label_set_markup(GTK_LABEL(ctlAbout.label_title), text);
      g_free(text);
    }

  if(semititle != NULL)
    {
      text = g_strdup_printf("<span size=\"large\" weight=\"heavy\">%s</span>", 
			     semititle);
      gtk_label_set_markup(GTK_LABEL(ctlAbout.label_semititle), text);
      g_free(text);
    }
    
  if(version != NULL)
    {
      text = g_strdup_printf("%s %s", 
			     _("Version"),
			     version);

      gtk_label_set_markup(GTK_LABEL(ctlAbout.label_version), text);

      if(text != NULL) g_free(text);
    }

  if(build != NULL)
    {
      text = g_strdup_printf("%s %s (for %s)", _("Build"), build, os);
      gtk_label_set_markup(GTK_LABEL(ctlAbout.label_build), text);
      g_free(text);
    }

  if(copyright != NULL)
    gtk_label_set_markup(GTK_LABEL(ctlAbout.label_copyright), copyright);
    
  if(license != NULL)
    gtk_label_set_markup(GTK_LABEL(ctlAbout.label_license), license);

  if(sn != NULL)
    gtk_label_set_markup(GTK_LABEL(ctlAbout.label_sn), sn);

  if((ani = gj_gtk_support_animation_new_from_file(FILE_JSF)) == NULL)
    {
      g_warning("%s: animation was NULL", __FUNCTION__);
      return FALSE;
    }

  /* set image */
  gtk_image_set_from_animation(GTK_IMAGE(ctlAbout.image_icon), ani);

  /* clean up */
  g_object_unref(G_OBJECT(ani));

  return TRUE;
}

GtkWidget *gj_gtk_about_credits_create_label()
{
  GtkWidget *label = NULL;
    
  label = gtk_label_new("");
  gtk_label_set_selectable(GTK_LABEL (label), TRUE);
  gtk_misc_set_alignment(GTK_MISC (label), 0.0, 0.0);
    
  gtk_misc_set_padding(GTK_MISC (label), 8, 8);
    
    
  gtk_label_set_justify(GTK_LABEL (label), GTK_JUSTIFY_LEFT);
    
  return label;
}

void gj_gtk_about_credits_update_authors(GtkWidget *label)
{
  GString *string = NULL;
  GSList *list = NULL;
  gchar *tmp = NULL;
    
  if(ctlAbout.authors == NULL) 
    {
      gtk_widget_hide(label);
      return;
    }
  else 
    {
      gtk_widget_show(label);
    }
    
  string = g_string_new(NULL);
    
  for(list=ctlAbout.authors; list; list=list->next) 
    {
      tmp = g_markup_escape_text(list->data, -1);
      g_string_append(string, tmp);
	
      if(list->next)
	g_string_append_c (string, '\n');
	
      g_free(tmp);
    }
    
  gtk_label_set_text(GTK_LABEL (label), string->str);
  g_string_free(string, TRUE);
}

void gj_gtk_about_credits_update_documenters(GtkWidget *label)
{
  GString *string = NULL;
  GSList *list = NULL;
  gchar *tmp = NULL;
                                                                                                                   
  if(ctlAbout.documenters == NULL)
    {
      gtk_widget_hide(label);
      return;
    }
  else
    {
      gtk_widget_show(label);
    }
                                                                                                                   
  string = g_string_new(NULL);
                                                                                                                   
  for(list=ctlAbout.documenters; list; list=list->next)
    {
      tmp = g_markup_escape_text(list->data, -1);
      g_string_append(string, tmp);
                                                                                                                   
      if(list->next)
	g_string_append (string, "\n");
                                                                                                                   
      g_free(tmp);
    }
                                                                                                                   
  gtk_label_set_text(GTK_LABEL (label), string->str);
  g_string_free(string, TRUE);
}

void gj_gtk_about_credits_update_translators(GtkWidget *label)
{
  GString *string = NULL;
  GSList *list = NULL;
  gchar *tmp = NULL;
    
  if(ctlAbout.translators == NULL) 
    {
      gtk_widget_hide(label);
      return;
    }
  else 
    {
      gtk_widget_show(label);
    }
    
  string = g_string_new(NULL);
    
  for(list=ctlAbout.translators; list; list=list->next) 
    {
      tmp = g_markup_escape_text(list->data, -1); 
      g_string_append(string, tmp);
	
      if(list->next)
	g_string_append (string, "\n");
	
      g_free(tmp); 
    }
    
  gtk_label_set_text(GTK_LABEL (label), string->str);
  g_string_free(string, TRUE);
}

gboolean gj_gtk_about_credits()
{
  GtkWidget *notebook = NULL;
  GtkWidget *dialog = NULL;
  GtkWidget *label = NULL;
  GtkWidget *sw = NULL;

  if(ctlAbout.credits != NULL)
    {
      gtk_window_present(GTK_WINDOW(ctlAbout.credits));
      return TRUE;
    }

  dialog = gtk_dialog_new_with_buttons (_("Credits"),
					GTK_WINDOW(ctlAbout.window),
					GTK_DIALOG_DESTROY_WITH_PARENT,
					GTK_STOCK_OK, GTK_RESPONSE_OK,
					NULL);
    
  ctlAbout.credits = dialog;
  gtk_window_set_default_size(GTK_WINDOW(dialog), 360, 260);
  gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
    
  g_signal_connect (dialog, "response",
		    G_CALLBACK(gtk_widget_destroy), dialog);
    
  g_signal_connect (dialog, "destroy",
		    G_CALLBACK(gtk_widget_destroyed),
		    &(ctlAbout.credits));
    
  notebook = gtk_notebook_new();
  gtk_container_set_border_width(GTK_CONTAINER(notebook), 8);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), notebook, TRUE, TRUE, 0);
    
  if(ctlAbout.authors != NULL) 
    {
      g_message("%s: configuring authors...", __FUNCTION__);
      label = gj_gtk_about_credits_create_label ();
	
      sw = gtk_scrolled_window_new(NULL, NULL);
      gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
				     GTK_POLICY_AUTOMATIC,
				     GTK_POLICY_AUTOMATIC);
      gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(sw), label);
      gtk_viewport_set_shadow_type(GTK_VIEWPORT(GTK_BIN(sw)->child), GTK_SHADOW_NONE);
	
      gtk_notebook_append_page(GTK_NOTEBOOK(notebook), sw,
			       gtk_label_new_with_mnemonic(_("Written by")));

      gj_gtk_about_credits_update_authors(label);
    }
    
  if(ctlAbout.documenters != NULL) 
    {
      g_message("%s: configuring documenters...", __FUNCTION__);
      label = gj_gtk_about_credits_create_label();
	
      sw = gtk_scrolled_window_new(NULL, NULL);
      gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
				     GTK_POLICY_AUTOMATIC,
				     GTK_POLICY_AUTOMATIC);
      gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(sw), label);
      gtk_viewport_set_shadow_type(GTK_VIEWPORT(GTK_BIN(sw)->child), GTK_SHADOW_NONE);
	
      gtk_notebook_append_page(GTK_NOTEBOOK(notebook), sw,
			       gtk_label_new_with_mnemonic(_("Documented by")));
	
      gj_gtk_about_credits_update_documenters(label);
    }
    
  if(ctlAbout.translators != NULL)  
    { 
      g_message("%s: configuring translators...", __FUNCTION__);
      label = gj_gtk_about_credits_create_label(); 
	
      sw = gtk_scrolled_window_new(NULL, NULL); 
      gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
				     GTK_POLICY_AUTOMATIC, 
				     GTK_POLICY_AUTOMATIC); 
      gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(sw), label);
      gtk_viewport_set_shadow_type(GTK_VIEWPORT(GTK_BIN(sw)->child), GTK_SHADOW_NONE); 
	
      gtk_notebook_append_page(GTK_NOTEBOOK(notebook), sw, 
			       gtk_label_new_with_mnemonic (_("Translated by"))); 
	
      gj_gtk_about_credits_update_translators(label); 
    } 
   
  gtk_widget_show_all(dialog);
    
  return TRUE;
}


/*
 * actual gui functions
 */
void on_about_response(GtkDialog *dialog, gint response, gpointer user_data)
{
  switch(response) 
    {
    case ABOUT_RESPONSE_CREDITS:
      gj_gtk_about_credits();
      return;
      break;

    case GTK_RESPONSE_OK:
    case GTK_RESPONSE_HELP:
    case GTK_RESPONSE_NONE:
    case GTK_RESPONSE_REJECT:
    case GTK_RESPONSE_ACCEPT:
    case GTK_RESPONSE_DELETE_EVENT:
    case GTK_RESPONSE_CANCEL:
    case GTK_RESPONSE_CLOSE:
    case GTK_RESPONSE_YES:
    case GTK_RESPONSE_NO:
    case GTK_RESPONSE_APPLY:
      break;
    }

  gj_gtk_about_unload();
}


#ifdef __cplusplus
}
#endif
