/* 
 * $RCSfile: gj_gtk_event_viewer.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/08/23 11:17:26 $
 * 
 * $Revision: 1.23 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "common.h"

#include "gj_main.h"
#include "gj_event.h"
#include "gj_presence.h"
#include "gj_message.h"
#include "gj_translate.h"
#include "gj_support.h"

#include "gj_gtk_event_viewer.h"
#include "gj_gtk_roster.h"


/* columns */
enum
  { 
    COL_EVENT_IMAGE,
    COL_EVENT_JID,
    COL_EVENT_NAME,
    COL_EVENT_ID,
    COL_EVENT_TYPE,
    COL_EVENT_TYPE_STR,
    COL_EVENT_DETAILS,
    COL_EVENT_TIME,
    COL_EVENT_NEW,
    COL_EVENT_COUNT
  };


struct 
{
  GtkWidget *window;
  
  GtkWidget *treeview;
  GtkWidget *checkbutton_show_jid;

} ctlEV;


gboolean gj_gtk_ev_setup();

/*
 * model functions 
 */
gboolean gj_gtk_ev_model_setup();
gboolean gj_gtk_ev_model_populate_columns(GtkTreeView *view);
gboolean gj_gtk_ev_model_get_selection(gchar **jid);
gboolean gj_gtk_ev_model_details_from_event(event ev, gchar **details, gboolean *warning);

/* gui callbacks */
void on_checkbutton_ev_show_jid_toggled(GtkToggleButton *togglebutton, gpointer data);

gboolean on_event_viewer_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);

/*
 * code
 */
gboolean gj_gtk_ev_load(gboolean show)
{
  GladeXML *xml = NULL;

  if(ctlEV.window != NULL)
    {
      gj_gtk_roster_set_status_event_error(FALSE);
      gtk_window_present(GTK_WINDOW(ctlEV.window));

      /* if we do it on load, the window is hidden, and we get warnings */
      gdk_window_set_decorations (GDK_WINDOW(ctlEV.window->window), 
				  GDK_DECOR_ALL);

      return FALSE;
    }

  if((xml = glade_xml_new(gnome_jabber_get_glade_file(),
			  "event_viewer",
			  NULL)) == NULL)
    {
      g_warning("gj_gtk_ev_load: couldn't open glade file at root 'event_viewer'");
      return FALSE;
    }
  
  ctlEV.window = glade_xml_get_widget(xml,"event_viewer");

  ctlEV.treeview = glade_xml_get_widget(xml,"treeview_ev");
  ctlEV.checkbutton_show_jid = glade_xml_get_widget(xml, "checkbutton_ev_show_jid");

#ifndef WIN32
  glade_xml_signal_autoconnect(xml);
#else
  g_signal_connect ((gpointer) ctlEV.window, "delete_event",
		    G_CALLBACK (on_event_viewer_delete_event),
		    NULL);
  g_signal_connect ((gpointer) ctlEV.checkbutton_show_jid, "toggled",
		    G_CALLBACK (on_checkbutton_ev_show_jid_toggled),
		    NULL);
#endif

  /* setup controls */
  gj_gtk_ev_setup();

  if(show == TRUE)
    gtk_window_present(GTK_WINDOW(ctlEV.window));

  gj_gtk_roster_set_status_event_error(FALSE);

  /* clean up */
  if(xml != NULL)
    {
      g_object_unref(G_OBJECT(xml));
      xml = NULL;
    }

  return TRUE;
}

gboolean gj_gtk_ev_unload()
{
  gtk_widget_hide(GTK_WIDGET(ctlEV.window));

  return TRUE;
}

gboolean gj_gtk_ev_setup()
{
  /* set up list */
  gj_gtk_ev_model_setup();

  return TRUE;
}

/*
 * model functions 
 */
gboolean gj_gtk_ev_model_setup()
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkListStore *store = NULL;
  GtkTreeSelection *selection = NULL;

  if((view = GTK_TREE_VIEW(ctlEV.treeview)) == NULL)
    {
      g_warning("gj_gtk_ev_model_setup: view was NULL");
      return FALSE;
    }

  if((selection = GTK_TREE_SELECTION(gtk_tree_view_get_selection(view))) == NULL)
    {
      g_warning("gj_gtk_ev_model_setup: selection was NULL");
      return FALSE;
    }

  /* create store */
  store = gtk_list_store_new(COL_EVENT_COUNT,
			     G_TYPE_STRING,    /* stock_id */
			     G_TYPE_STRING,    /* jid */
			     G_TYPE_STRING,    /* name - friendly version */
			     G_TYPE_INT,       /* event id */
			     G_TYPE_INT,       /* type */
			     G_TYPE_STRING,    /* type string */
			     G_TYPE_STRING,    /* details */	
			     G_TYPE_STRING,    /* time */
			     G_TYPE_BOOLEAN);  /* new */

  /* model */
  model  = GTK_TREE_MODEL(store);
  gtk_tree_view_set_model(view, model);

  /* selection */
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);

  /* columns */
  gj_gtk_ev_model_populate_columns(view);
  
  /* properties */
  gtk_tree_view_set_enable_search(view, TRUE);
  gtk_tree_view_set_search_column(view, COL_EVENT_TIME); 
  gtk_tree_view_set_rules_hint(view, FALSE);
  gtk_tree_view_set_headers_visible(view, TRUE);
  gtk_tree_view_set_headers_clickable(view, TRUE);

  /* clean up */
  g_object_unref(G_OBJECT(store));

  return TRUE;
}

gboolean gj_gtk_ev_model_populate_columns(GtkTreeView *view)
{
  GtkTreeViewColumn *column = NULL; 
  GtkCellRenderer *renderer = NULL;
  GtkCellRenderer *renderer_image = NULL;
  GtkCellRenderer *renderer_text = NULL;
  guint col_offset = 0;

  if(GTK_TREE_VIEW(view) == NULL)
    {
      g_warning("gj_gtk_ev_model_populate_columns: view was NULL");
      return FALSE;
    }  

  /* cell renderer's */
  renderer_image = gtk_cell_renderer_pixbuf_new();
  renderer_text = gtk_cell_renderer_text_new();

  column = gtk_tree_view_column_new();
  gtk_tree_view_column_set_title (column, _("Name"));

  /* COL_EVENT_IMAGE */
  g_object_set(G_OBJECT(renderer_image), "yalign", (gfloat)0, NULL);
  gtk_tree_view_column_pack_start(column, renderer_image, FALSE);
  gtk_tree_view_column_add_attribute(column,
				     renderer_image,
				     "stock_id",COL_EVENT_IMAGE);  

  /* COL_EVENT_NAME */
  g_object_set(G_OBJECT(renderer_text), "yalign", (gfloat)0, NULL);
  gtk_tree_view_column_pack_end(column, renderer_text, TRUE);
  gtk_tree_view_column_add_attribute(column,
				     renderer_text,
				     "text",COL_EVENT_NAME);

  /* insert column */
  gtk_tree_view_insert_column(GTK_TREE_VIEW(view), column, -1);

  gtk_tree_view_column_set_sort_column_id(column, COL_EVENT_NAME);
  gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN(column), TRUE);
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);

  /* COL_EVENT_JID */
  renderer = gtk_cell_renderer_text_new();
  col_offset = gtk_tree_view_insert_column_with_attributes(view,
							   -1, "JID",
							   renderer, 
							   "text", COL_EVENT_JID,
							   NULL);
  
  g_object_set(G_OBJECT(renderer), "yalign", (gfloat)0, NULL);
  g_object_set_data(G_OBJECT(renderer), "column", (gint*)COL_EVENT_JID);

  column = gtk_tree_view_get_column(view, col_offset - 1);
  gtk_tree_view_column_set_sort_column_id(column, COL_EVENT_JID);
  gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN(column), TRUE);
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
  gtk_tree_view_column_set_visible(GTK_TREE_VIEW_COLUMN(column), FALSE);

  /* COL_EVENT_TIME */
  renderer = gtk_cell_renderer_text_new();
  col_offset = gtk_tree_view_insert_column_with_attributes(view,
							   -1, _("Time"),
							   renderer, 
							   "text", COL_EVENT_TIME,
							   NULL);
  
  g_object_set(G_OBJECT(renderer), "yalign", (gfloat)0, NULL);
  g_object_set_data(G_OBJECT(renderer), "column", (gint*)COL_EVENT_TIME);

  column = gtk_tree_view_get_column(view, col_offset - 1);
  gtk_tree_view_column_set_sort_column_id(column, COL_EVENT_ID);
  gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN(column), TRUE);

  /* COL_EVENT_TYPE_STR */
  renderer = gtk_cell_renderer_text_new();
  col_offset = gtk_tree_view_insert_column_with_attributes(view,
							   -1, _("Type"),
							   renderer, 
							   "text", COL_EVENT_TYPE_STR,
							   NULL);
  
  g_object_set(G_OBJECT(renderer), "yalign", (gfloat)0, NULL);
  g_object_set_data(G_OBJECT(renderer), "column", (gint*)COL_EVENT_TYPE_STR);

  column = gtk_tree_view_get_column(view, col_offset - 1);
  gtk_tree_view_column_set_sort_column_id(column, COL_EVENT_TYPE_STR);
  gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN(column), TRUE);

  /* COL_EVENT_DETAILS */
  renderer = gtk_cell_renderer_text_new();
  col_offset = gtk_tree_view_insert_column_with_attributes(view,
							   -1, _("Details"),
							   renderer, 
							   "markup", COL_EVENT_DETAILS,
							   NULL);
  
  g_object_set(G_OBJECT(renderer), "yalign", (gfloat)0, NULL);
  g_object_set_data(G_OBJECT(renderer), "column", (gint*)COL_EVENT_DETAILS);

  column = gtk_tree_view_get_column(view, col_offset - 1);
  gtk_tree_view_column_set_sort_column_id(column, COL_EVENT_DETAILS);
  gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN(column), TRUE);
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);

  return TRUE;
}

gboolean gj_gtk_ev_model_get_selection(gchar **jid)
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkTreeSelection *selection = NULL;
  GtkTreeIter iter;
  
  if(jid == NULL)
    {
      g_warning("gj_gtk_ev_model_get_selection: jid pointer passed was NULL");
      return FALSE;
    }

  if((view = GTK_TREE_VIEW(ctlEV.treeview)) == NULL)
    {
      g_warning("gj_gtk_ev_model_get_selection: view was NULL");
      return FALSE;
    }

  if((model = GTK_TREE_MODEL(gtk_tree_view_get_model(view))) == NULL)
    {
      g_warning("gj_gtk_ev_model_get_selection: model was NULL");
      return FALSE;
    }

  if((selection = GTK_TREE_SELECTION(gtk_tree_view_get_selection(view))) == NULL)
    {
      g_warning("gj_gtk_ev_model_get_selection: selection was NULL");
      return FALSE;
    }

  if(gtk_tree_selection_get_selected(selection, &model, &iter) == FALSE)
    return FALSE;
  
  gtk_tree_model_get(model, &iter, 
		     COL_EVENT_JID, jid,
		     -1);     

  return TRUE;
}

gboolean gj_gtk_ev_model_add_from_event(event ev)
{
  GtkTreeView *view = NULL;
  GtkListStore *store = NULL;
  GtkTreeIter iter;

  gchar *stock_id = NULL;
  gchar *details = NULL;
  gboolean warning = FALSE;

  gchar *name = NULL;
  gchar *groups = NULL;
  rosterItem ri = NULL;

  if(ev == NULL)
    {
      g_warning("%s: ev was NULL", __FUNCTION__);
      return FALSE;
    }

  if((view = GTK_TREE_VIEW(ctlEV.treeview)) == NULL)
    {
      g_warning("%s: view was NULL", __FUNCTION__);
      return FALSE;
    }

  if((store = GTK_LIST_STORE(gtk_tree_view_get_model(view))) == NULL)
    {
      g_warning("%s: store was NULL", __FUNCTION__);
      return FALSE;
    }

  if((ri = gj_event_get_roster_item(ev)) == NULL)
    {
      g_warning("%s: could not get roster item from event id:%d", __FUNCTION__, gj_event_get_id(ev));
      return FALSE;
    }
  
  if(gj_gtk_ev_model_details_from_event(ev, &details, &warning) == TRUE)
    {
      /* set pb */
      if(warning == FALSE)
	stock_id = GTK_STOCK_DIALOG_INFO;
      else
	stock_id = GTK_STOCK_DIALOG_ERROR;

      if(warning == TRUE)
	gj_gtk_roster_set_status_event_error(TRUE);	
    }
  else
    g_warning("%s: failed to get details, event may be incomplete.", __FUNCTION__);


  groups = gj_roster_item_get_groups_as_string(ri);
  name = g_strdup_printf("%s %s%s%s",
			  gj_roster_item_get_name(ri),
			  groups?"[":"",
			  groups?groups:"",
			  groups?"]":"");
  
  /* add */ 
  gtk_list_store_append(store, &iter);
  gtk_list_store_set(store, &iter,    
		     COL_EVENT_IMAGE, stock_id,
		     COL_EVENT_JID, gj_event_get_jid(ev),
		     COL_EVENT_NAME, name,
		     COL_EVENT_ID, gj_event_get_id(ev),
		     COL_EVENT_TYPE, gj_event_get_type(ev),
		     COL_EVENT_TYPE_STR, gj_translate_event_type(gj_event_get_type(ev)), 
		     COL_EVENT_DETAILS, details,
		     COL_EVENT_TIME, gj_support_get_timestamp_from_timet(gj_event_get_time(ev)),
		     COL_EVENT_NEW, TRUE,
		     -1);

  /* clean up */
  if(groups != NULL) g_free(groups);
  if(name != NULL) g_free(name);
  if(details != NULL) g_free(details);
/*   if(pb != NULL) g_object_unref(pb); */
    
  return TRUE;
}

gboolean gj_gtk_ev_model_details_from_event(event ev, gchar **details, gboolean *warning)
{
  presence pres = NULL;
  message msg = NULL;

  if(ev == NULL)
    {
      g_warning("%s: event was NULL", __FUNCTION__);
      return FALSE;
    }
  
  if(details == NULL)
    {
      g_warning("%s: details pointer was NULL", __FUNCTION__);
      return FALSE;
    }

  if(warning == NULL)
    {
      g_warning("%s: warning pointer was NULL", __FUNCTION__);
      return FALSE;
    }

  switch(gj_event_get_type(ev))
    {
    case eventTypePresence:
      {
	gchar *type = NULL;
	gchar *show = NULL;
	gchar *status = NULL;

	gchar *error_code = NULL;
	gchar *error_reason = NULL;

	if((pres = gj_event_get_data(ev)) == NULL)
	  {
	    g_warning("%s: presence was NULL", __FUNCTION__);
	    return FALSE;
	  }
	
	if(gj_presence_get_type(pres) == presenceTypeError)
	  *warning = TRUE;
	else
	  *warning = FALSE;

	/* check for markup problems */
	if(gj_translate_presence_type(gj_presence_get_type(pres)) != NULL)
	  type = g_markup_escape_text(gj_translate_presence_type(gj_presence_get_type(pres)), -1);

	if(gj_translate_presence_show(gj_presence_get_show(pres)) != NULL)
	  show = g_markup_escape_text(gj_translate_presence_show(gj_presence_get_show(pres)), -1);

	if(gj_presence_get_status(pres) != NULL) 
 	  status = g_markup_escape_text(gj_presence_get_status(pres), -1);
	
	if(gj_presence_get_error_code(pres) != NULL)
	  error_code = g_markup_escape_text(gj_presence_get_error_code(pres), -1);
	
	if(gj_presence_get_error_reason(pres) != NULL)
	  error_reason = g_markup_escape_text(gj_presence_get_error_reason(pres), -1);

	
	*details = g_strdup_printf("%s:<b>%s</b>, %s:<b>%s</b>%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s", 
				   _("type"),
				   type,
				   _("show"), 
				   show,
				   status!=NULL?", ":"",
				   status!=NULL?_("status"):"",
				   status!=NULL?":<b>":"",
				   status!=NULL?status:"",
				   status!=NULL?"</b>":"",
				   error_code != NULL?" ,":"",
				   error_code != NULL?_("error code"):"",
				   error_code != NULL?":<b>":"",
				   error_code != NULL?error_code:"",
				   error_code != NULL?"</b>":"",
				   error_reason != NULL?", ":"",
				   error_reason != NULL?_("error reason"):"",
				   error_reason != NULL?":<b>":"",
				   error_reason != NULL?error_reason:"",
				   error_reason != NULL?"</b>":"");


	if(type != NULL) g_free(type);
	if(show != NULL) g_free(show);
	if(status != NULL) g_free(status);

	if(error_code != NULL) g_free(error_code);
	if(error_reason != NULL) g_free(error_reason);
      }
	
      break;

    case eventTypeMessage:
      {
	gchar *type = NULL;
	gchar *body = NULL;
	gchar *thread = NULL;

	gchar *xevent = NULL;

	gchar *error_code = NULL;
	gchar *error_reason = NULL;

	if((msg = gj_event_get_data(ev)) == NULL)
	  {
	    g_warning("%s: message was NULL", __FUNCTION__);
	    return FALSE;
	  }
	
	if(gj_message_get_type(msg) == messageTypeError)
	  *warning = TRUE;
	else
	  *warning = FALSE; 

	/* FIXME: should really check all strings before parsing them to a markup function */
	if(gj_translate_message_type(gj_message_get_type(msg)) != NULL)
	  type = g_markup_escape_text(gj_translate_message_type(gj_message_get_type(msg)), -1);

	if(gj_message_get_body(msg) != NULL)
	  body = g_markup_escape_text(gj_message_get_body(msg), -1);
	
	if(gj_message_get_thread(msg) != NULL)
	  thread = g_markup_escape_text(gj_message_get_thread(msg), -1);
	
	if(gj_message_get_x_event_type(msg) != 0 && 
	   gj_translate_message_x_event_type(gj_message_get_x_event_type(msg)) != NULL)
	  xevent = g_markup_escape_text(gj_translate_message_x_event_type(gj_message_get_x_event_type(msg)), -1);

	if(gj_message_get_error_code(msg) != NULL)
	  error_code = g_markup_escape_text(gj_message_get_error_code(msg), -1);
	
	if(gj_message_get_error_reason(msg) != NULL)
	  error_reason = g_markup_escape_text(gj_message_get_error_reason(msg), -1);


	*details = g_strdup_printf("%s:<b>%s</b>%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s", 
				   _("type"),
				   type, 
				   thread != NULL?", ":"",
				   thread != NULL?_("thread"):"",
				   thread != NULL?":<b>":"",
				   thread != NULL?thread:"",
				   thread != NULL?"</b>":"",
				   xevent != NULL?", ":"", 
				   xevent != NULL?_("xevent"):"", 
				   xevent != NULL?":<b>":"", 
				   xevent != NULL?xevent:"",
				   xevent != NULL?"</b>":"",
				   error_code != NULL?", ":"",
				   error_code != NULL?_("error code"):"",
				   error_code != NULL?":<b>":"",
				   error_code != NULL?error_code:"",
				   error_code != NULL?"</b>":"",
				   error_reason != NULL?", ":"",
				   error_reason != NULL?_("error reason"):"",
				   error_reason != NULL?":<b>":"",
				   error_reason != NULL?error_reason:"",
				   error_reason != NULL?"</b>":"",
				   warning && body != NULL?", ":"",
				   warning && body != NULL?_("body"):"",
				   warning && body != NULL?":\n<b>":"",
				   warning && body != NULL?body:"", 
				   warning && body != NULL?"</b>":"");

	if(type != NULL) g_free(type);
	if(body != NULL) g_free(body);
	if(thread != NULL) g_free(thread);

	if(xevent != NULL) g_free(xevent);

	if(error_code != NULL) g_free(error_code);
	if(error_reason != NULL) g_free(error_reason);

      }
      break;

    case eventTypeFile:
    default:
      break;
    }
  
  return TRUE;
}

/* 
 * gui callbacks
 */
void on_checkbutton_ev_show_jid_toggled(GtkToggleButton *togglebutton, gpointer data)
{
  GtkTreeView *view = NULL;
  GtkTreeViewColumn *column = NULL;

  gboolean visible = FALSE;

  if((view = GTK_TREE_VIEW(ctlEV.treeview)) == NULL)
    {
      g_warning("%s: could not get tree view", __FUNCTION__);
      return;
    }

  visible = gtk_toggle_button_get_active(togglebutton);
  column = gtk_tree_view_get_column(view, 1);

  if(column == NULL)
    {
      g_warning("%s: could not get column", __FUNCTION__);
      return;
    }

  gtk_tree_view_column_set_visible(GTK_TREE_VIEW_COLUMN(column), visible);
}

gboolean on_event_viewer_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  gj_gtk_ev_unload();

  return TRUE;
}

#ifdef __cplusplus
}
#endif
