/* 
 * $RCSfile: gj_gtk_my_presence.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/07/31 16:43:54 $
 * 
 * $Revision: 1.12 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "common.h"

#include "gj_main.h"
#include "gj_presence.h"
#include "gj_me.h"
#include "gj_support.h"

#include "gj_gtk_roster.h"
#include "gj_gtk_my_presence.h"
#include "gj_gtk_support.h"

struct 
{
  GtkWidget *window;

  GtkWidget *optionmenu_show;
  GtkWidget *spinbutton_priority;
  GtkWidget *textview_status;
  
  GtkWidget *image_presence;
  GtkWidget *label_presence;

  GtkWidget *button_ok;
  GtkWidget *button_cancel;

} ctlMP;


struct
{
  presenceType type;
  presenceShow show;

} optMP;
  

gboolean gj_gtk_mp_setup();

/* 
 * member functions 
 */

/* sets */
gboolean gj_gtk_mp_set_type(presenceType type);
gboolean gj_gtk_mp_set_show(presenceShow show);

/* gets */
presenceType gj_gtk_mp_get_type();
presenceShow gj_gtk_mp_get_show();


/* 
 * gui callbacks 
*/
void on_mnu_mp_normal_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_mnu_mp_chat_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_mnu_mp_away_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_mnu_mp_xa_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_mnu_mp_dnd_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_mnu_mp_unavailable_activate(GtkMenuItem *menuitem, gpointer user_data);

void on_button_mp_ok_clicked(GtkButton *button, gpointer user_data);
void on_button_mp_cancel_clicked(GtkButton *button, gpointer user_data);

gboolean on_my_presence_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);

/*
 * code
 */
gboolean gj_gtk_mp_load(presenceType type, presenceShow show)
{
  GladeXML *xml = NULL;

  if((xml = glade_xml_new(gnome_jabber_get_glade_file(),
			  "my_presence",
			  NULL)) == NULL)
    {
      g_warning("gj_gtk_mp_load: couldn't open glade file at root 'my_presence'");
      return FALSE;
    }
  
  ctlMP.window = glade_xml_get_widget(xml,"my_presence");

  ctlMP.optionmenu_show = glade_xml_get_widget(xml,"optionmenu_mp_show");
  ctlMP.spinbutton_priority = glade_xml_get_widget(xml,"spinbutton_mp_priority");
  ctlMP.textview_status = glade_xml_get_widget(xml,"textview_mp_status");

  ctlMP.image_presence = glade_xml_get_widget(xml,"image_mp_presence");
  ctlMP.label_presence = glade_xml_get_widget(xml,"label_mp_presence");

  ctlMP.button_ok = glade_xml_get_widget(xml,"button_mp_ok");
  ctlMP.button_cancel = glade_xml_get_widget(xml,"button_mp_cancel");
  
  gdk_window_set_decorations (GDK_WINDOW(ctlMP.window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE | GDK_DECOR_MENU);

#ifndef WIN32
  glade_xml_signal_autoconnect(xml);
#else
  g_signal_connect ((gpointer) ctlMP.window, "delete_event",
		    G_CALLBACK (on_my_presence_delete_event),
		    NULL);
  g_signal_connect ((gpointer) ctlMP.button_cancel, "clicked",
		    G_CALLBACK (on_button_mp_cancel_clicked),
		    NULL);
  g_signal_connect ((gpointer) ctlMP.button_ok, "clicked",
		    G_CALLBACK (on_button_mp_ok_clicked),
		    NULL);
#endif

  /* set defaults */
  if(type <= 0)
    gj_gtk_mp_set_type(presenceTypeAvailable);
  else
    gj_gtk_mp_set_type(type);

  if(show <= 0)
    gj_gtk_mp_set_show(presenceShowNormal);
  else
    gj_gtk_mp_set_show(show);

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ctlMP.spinbutton_priority), 
			    (gint8)gj_me_get_presence_priority());
  
  gj_gtk_mp_setup();

  /* clean up */
  if(xml != NULL)
    {
      g_object_unref(G_OBJECT(xml));
      xml = NULL;
    }

  return TRUE;
}

gboolean gj_gtk_mp_unload()
{
  if(ctlMP.window == NULL)
    return TRUE;

  gtk_widget_destroy(ctlMP.window);

  ctlMP.window = NULL;

  ctlMP.optionmenu_show = NULL;
  ctlMP.spinbutton_priority = NULL;
  ctlMP.textview_status = NULL;

  ctlMP.image_presence = NULL;
  ctlMP.label_presence = NULL;

  ctlMP.button_ok = NULL;
  ctlMP.button_cancel = NULL;
  
  return TRUE;
}

gboolean gj_gtk_mp_setup()
{
  gchar *file = NULL;
  gchar *text = NULL;
  gchar *status = NULL;

  GdkPixbuf *pixbuf = NULL;

  if(gj_gtk_mp_get_type() != presenceTypeAvailable)
    {
      switch(gj_gtk_mp_get_type())
	{
	case presenceTypeUnAvailable:
	  file = FILE_PRESENCE_UNAVAILABLE;
	  text = g_strdup_printf(_("Unavailable"));
	  break;
	default:
	  g_warning("%s: failed to determin presence 'type' from %d", __FUNCTION__, gj_gtk_mp_get_type());
	  return FALSE;
	}
    }
  else
    {
      switch(gj_gtk_mp_get_show())
	{
	case presenceShowAway:
	  file = FILE_PRESENCE_AWAY;
	  text = g_strdup_printf(_("Away"));
	  status = g_strdup_printf(_("Away (be right back)"));
	  break;
	case presenceShowChat:
	  file = FILE_PRESENCE_CHAT;
	  text = g_strdup_printf(_("Chat"));
	  status = g_strdup_printf(_("Chat (open to talk...)"));
	  break;
	case presenceShowDND:
	  file = FILE_PRESENCE_DND;
	  text = g_strdup_printf(_("Do Not Disturb"));
	  status = g_strdup_printf(_("DND (do not disturb!)"));
	  break;
	case presenceShowNormal:
	  file = FILE_PRESENCE_NORMAL;
	  text = g_strdup_printf(_("Normal"));
	  status = g_strdup_printf(_("Normal (available)"));
	  break;
	case presenceShowXA:
	  file = FILE_PRESENCE_XA;
	  text = g_strdup_printf(_("eXtended Away"));
	  status = g_strdup_printf(_("eXtended Away (not available right now)"));
	  break;
	default:
	  g_warning("%s: failed to determin presence 'show' from %d", __FUNCTION__, gj_gtk_mp_get_show());
	  return FALSE;
	}
    }

  pixbuf = gj_gtk_support_pixbuf_new_from_file(file);
  text = g_strconcat("<b>",text,"</b>",NULL);

  gtk_image_set_from_pixbuf(GTK_IMAGE(ctlMP.image_presence),pixbuf);
  gtk_label_set_markup(GTK_LABEL(ctlMP.label_presence),text);

  if(status != NULL)
    {
      GtkTextBuffer *buffer = NULL;
      
      buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ctlMP.textview_status));
      gtk_text_buffer_set_text(buffer, status, -1);
      
      /* select all text ??? how? */
    }
  
  /* clean up */
  if(text != NULL) g_free(text);
  if(status != NULL) g_free(status);
  if(pixbuf != NULL) g_object_unref(G_OBJECT(pixbuf));

  return TRUE;
}

/* 
 * member functions 
 */

/* sets */
gboolean gj_gtk_mp_set_type(presenceType type)
{
  optMP.type = type;
  return TRUE;
}

gboolean gj_gtk_mp_set_show(presenceShow show)
{
  optMP.show = show;
  return TRUE;
}

/* gets */
presenceType gj_gtk_mp_get_type()
{
  return optMP.type;
}

presenceShow gj_gtk_mp_get_show()
{
  return optMP.show;
}

/*
 * GUI events
 */
void on_mnu_mp_normal_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  gj_gtk_mp_set_type(presenceTypeAvailable);
  gj_gtk_mp_set_show(presenceShowNormal);

  gj_gtk_mp_setup();
}

void on_mnu_mp_chat_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  gj_gtk_mp_set_type(presenceTypeAvailable);
  gj_gtk_mp_set_show(presenceShowChat);

  gj_gtk_mp_setup();
}

void on_mnu_mp_away_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  gj_gtk_mp_set_type(presenceTypeAvailable);
  gj_gtk_mp_set_show(presenceShowAway);

  gj_gtk_mp_setup();
}

void on_mnu_mp_xa_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  gj_gtk_mp_set_type(presenceTypeAvailable);
  gj_gtk_mp_set_show(presenceShowXA);

  gj_gtk_mp_setup();
}

void on_mnu_mp_dnd_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  gj_gtk_mp_set_type(presenceTypeAvailable);
  gj_gtk_mp_set_show(presenceShowDND);

  gj_gtk_mp_setup();
}

void on_mnu_mp_unavailable_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  gj_gtk_mp_set_type(presenceTypeUnAvailable);
  gj_gtk_mp_set_show(0);

  gj_gtk_mp_setup();
}

void on_button_mp_ok_clicked(GtkButton *button, gpointer user_data)
{
  GtkTextBuffer *buffer = NULL;
  GtkTextIter bufStart;
  GtkTextIter bufStop;

  gchar *status = NULL;
  gint8 priority = 0;

  /* get priority */
  priority = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ctlMP.spinbutton_priority));

  /* get status */
  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ctlMP.textview_status));
  gtk_text_buffer_get_iter_at_offset(buffer, &bufStart, 0);
  gtk_text_buffer_get_iter_at_offset(buffer, &bufStop, -1);
  status = gtk_text_buffer_get_text(buffer,&bufStart,&bufStop,TRUE);

  if(status != NULL && g_utf8_strlen(status, -1) < 1)
    status = NULL;

  /* update myself */
  gj_gtk_roster_user_set_presence_and_save(gj_gtk_mp_get_type(), 
					   gj_gtk_mp_get_show(), 
					   status, 
					   priority);

  /* close up */
  gj_gtk_mp_unload();
}

void on_button_mp_cancel_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_mp_unload();
}

gboolean on_my_presence_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  gj_gtk_mp_unload();

  return TRUE;
}

#ifdef __cplusplus
}
#endif
