/*
 * $RCSfile: gj_gtk_register_new.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/07/31 16:43:55 $
 *
 * $Revision: 1.17 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* gui */
#include <glade/glade.h>
#include <gtk/gtk.h>

/* local */
#include "common.h"

#include "gj_main.h"
#include "gj_stream.h"
#include "gj_iq_requests.h"
#include "gj_parser.h"
#include "gj_me.h"

#include "gj_gtk_register_new.h"
#include "gj_gtk_support.h"
#include "gj_gtk_msg.h"
#include "gj_gtk_connection_settings.h"

/* local functions */
gboolean gj_gtk_rn_setup();

gboolean gj_gtk_rn_passwords_changed();
gboolean gj_gtk_rn_check();

gboolean gj_gtk_rn_save_settings();

gboolean gj_gtk_rn_connection();
void gj_gtk_rn_connected_response(stream stm, gint requestID);
void gj_gtk_rn_registered_response(guint id);

/*
 * actual GUI functions
 */
void on_entry_rn_password1_changed(GtkEditable *editable, gpointer user_data);
void on_entry_rn_password2_changed(GtkEditable *editable, gpointer user_data);

void on_button_rn_ok_clicked(GtkButton *button, gpointer user_data);
void on_button_rn_cancel_clicked(GtkButton *button, gpointer user_data);

gboolean on_register_new_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);

struct 
{
  /* search */
  GtkWidget *window;

  GtkWidget *button_ok;
  GtkWidget *button_cancel;

  GtkWidget *combo_server;
  GtkWidget *combo_entry_server;
  GtkWidget *spinbutton_port;
  GtkWidget *entry_username;  
  GtkWidget *entry_password1;
  GtkWidget *entry_password2;

  GtkWidget *checkbutton_use_configuration;

  GtkWidget *vbox;

} ctlRN;

struct
{
  stream stm;

} optRN;

/*
 * external functions...
 */
gboolean gj_gtk_rn_load()
{
  GladeXML *xml = NULL;

  /* load dialog */
  if((xml = glade_xml_new(gnome_jabber_get_glade_file(),
			  "register_new",
			  NULL)) == NULL)
    {
      g_warning("gj_gtk_rn_load: couldn't open glade file at root 'register_new'");
      return FALSE;
    }
  
  /* controls */
  ctlRN.window = glade_xml_get_widget(xml, "register_new");

  ctlRN.button_ok = glade_xml_get_widget(xml, "button_rn_ok");
  ctlRN.button_cancel = glade_xml_get_widget(xml, "button_rn_cancel");

  ctlRN.combo_server = glade_xml_get_widget(xml, "combo_rn_server");
  ctlRN.combo_entry_server = glade_xml_get_widget(xml, "combo-entry_rn_server");
  ctlRN.spinbutton_port = glade_xml_get_widget(xml, "spinbutton_rn_port");
  ctlRN.entry_username = glade_xml_get_widget(xml, "entry_rn_username");
  ctlRN.entry_password1 = glade_xml_get_widget(xml, "entry_rn_password1");  
  ctlRN.entry_password2 = glade_xml_get_widget(xml, "entry_rn_password2");

  ctlRN.checkbutton_use_configuration = glade_xml_get_widget(xml, "checkbutton_rn_use_configuration");

  ctlRN.vbox = glade_xml_get_widget(xml,"dialog-vbox");

  gdk_window_set_decorations (GDK_WINDOW(ctlRN.window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE);

#ifndef WIN32
  glade_xml_signal_autoconnect(xml);
#else
  g_signal_connect ((gpointer) ctlRN.window, "delete_event",
		    G_CALLBACK (on_register_new_delete_event),
		    NULL);
  g_signal_connect ((gpointer) ctlRN.entry_password1, "changed",
		    G_CALLBACK (on_entry_rn_password1_changed),
		    NULL);
  g_signal_connect ((gpointer) ctlRN.entry_password2, "changed",
		    G_CALLBACK (on_entry_rn_password2_changed),
		    NULL);
  g_signal_connect ((gpointer) ctlRN.button_cancel, "clicked",
		    G_CALLBACK (on_button_rn_cancel_clicked),
		    NULL);
  g_signal_connect ((gpointer) ctlRN.button_ok, "clicked",
		    G_CALLBACK (on_button_rn_ok_clicked),
		    NULL);
#endif

  gj_gtk_rn_setup();

  /* clean up */
  if(xml != NULL)
    {
      g_object_unref(G_OBJECT(xml));
      xml = NULL;
    }

  return TRUE;
}

gboolean gj_gtk_rn_unload()
{
  gtk_widget_destroy(ctlRN.window);

  return TRUE;
}

gboolean gj_gtk_rn_setup()
{
  GList *servers = NULL;
  const gchar **server_listing = NULL;
  const gchar *str = NULL;
  gint i = 0;

  server_listing = gnome_jabber_get_server_list();
  
  /* set a limit of a 1000 */
  while((str = server_listing[i++]) != NULL && i < 1000)
      servers = g_list_append(servers, (gchar*)str);

  gtk_combo_set_popdown_strings(GTK_COMBO(ctlRN.combo_server), servers);

  /* clean up */
  if(servers != NULL) g_list_free(servers);

  return TRUE;
}

gboolean gj_gtk_rn_passwords_changed()
{
  gboolean match = FALSE;

  gchar *password1 = NULL;
  gchar *password2 = NULL;

  password1 = gtk_editable_get_chars(GTK_EDITABLE(ctlRN.entry_password1),0,-1);
  password2 = gtk_editable_get_chars(GTK_EDITABLE(ctlRN.entry_password2),0,-1);

  if(password1 != NULL && g_utf8_strlen(password1, -1) > 0 && 
     password2 != NULL && g_utf8_strlen(password2, -1) > 0)
    {
      if(strcmp(password1,password2) == 0)
	match = TRUE;
    }
  else
    {
      gtk_widget_set_sensitive(ctlRN.button_ok,FALSE);
      return TRUE;
    }
  
  /* change gui based on match */
  if(match == TRUE)
    gtk_widget_set_sensitive(ctlRN.button_ok,TRUE);
  else
    gtk_widget_set_sensitive(ctlRN.button_ok,FALSE);

  return TRUE;
}

gboolean gj_gtk_rn_check()
{
  gboolean passed = FALSE;

  gchar *username = NULL;
  gchar *server = NULL;

  gchar *password1 = NULL;
  gchar *password2 = NULL;

  server = gtk_editable_get_chars(GTK_EDITABLE(ctlRN.combo_entry_server),0,-1);
  username = gtk_editable_get_chars(GTK_EDITABLE(ctlRN.entry_username),0,-1);

  password1 = gtk_editable_get_chars(GTK_EDITABLE(ctlRN.entry_password1),0,-1);
  password2 = gtk_editable_get_chars(GTK_EDITABLE(ctlRN.entry_password2),0,-1);

  /* do checks */
  if(server == NULL || g_utf8_strlen(server, -1) < 1)
    {
      /* server not entered */
      gj_gtk_msg_new(TRUE,
		     GJ_GTK_MSG_BUTTON_OK | GJ_GTK_MSG_ICON_EXCLAIMATION,
		     (MsgCb)NULL,
		     NULL,
		     _("Details incomplete."),
		     _("Server was not specified."),
		     NULL);

      passed = FALSE;
    }
  else if(username == NULL || g_utf8_strlen(username, -1) < 1)
    {
      /* username not entered */
      gj_gtk_msg_new(TRUE,
		     GJ_GTK_MSG_BUTTON_OK | GJ_GTK_MSG_ICON_EXCLAIMATION,
		     (MsgCb)NULL,
		     NULL,
		     _("Details incomplete."),
		     _("Username was not specified."),
		     NULL);

      passed = FALSE;
    }
  else if(password1 == NULL || password2 == NULL || g_utf8_strlen(password1, -1) < 1 || g_utf8_strlen(password2, -1) < 1)
    {
      /* passwords not entered */
      gj_gtk_msg_new(TRUE,
		     GJ_GTK_MSG_BUTTON_OK | GJ_GTK_MSG_ICON_EXCLAIMATION,
		     (MsgCb)NULL,
		     NULL,
		     _("Details incomplete."),
		     _("Password was not specified."),
		     NULL);

      passed = FALSE;
    }
  else if(g_strcasecmp(password1,password2) != 0)
    {
      /* passwords do not match */
      gj_gtk_msg_new(TRUE,
		     GJ_GTK_MSG_BUTTON_OK | GJ_GTK_MSG_ICON_EXCLAIMATION,
		     (MsgCb)NULL,
		     NULL,
		     _("Details incomplete."),
		     _("Passwords do NOT match."),
		     NULL);

      passed = FALSE;
    }
  else
    {
      /* ok */
      passed = TRUE;
    }

  /* clean up */
  if(server != NULL) g_free(server);
  if(username != NULL) g_free(username);
  if(password1 != NULL) g_free(password1);
  if(password2 != NULL) g_free(password2);

  return passed;
}

gboolean gj_gtk_rn_save_settings()
{
  gchar *server = NULL;
  gint16 port = 0;
  gchar *username = NULL;
  gchar *password = NULL;

  /* if checkbutton is selected, change the connection settings */
  server = gtk_editable_get_chars(GTK_EDITABLE(ctlRN.combo_entry_server),0,-1);
  port = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ctlRN.spinbutton_port));
  username = gtk_editable_get_chars(GTK_EDITABLE(ctlRN.entry_username),0,-1);
  password = gtk_editable_get_chars(GTK_EDITABLE(ctlRN.entry_password1),0,-1);

  /* set */
  gj_gtk_cs_set_server(server);
  gj_gtk_cs_set_port(port);
  gj_gtk_cs_set_username(username);
  gj_gtk_cs_set_password(password);

  /* save */
  gj_gtk_cs_save();

  /* clean up */
  if(server != NULL) g_free(server);
  if(username != NULL) g_free(username);
  if(password != NULL) g_free(password);

  return TRUE;
}

gboolean gj_gtk_rn_connection()
{
  stream stm = NULL;
  gchar *server = NULL;
  gint16 port = 0;

#if 0
  if(gj_me_get_is_connected() == TRUE)
    {
      g_message("gj_gtk_rn_connection: can not register while still connected!");
      return FALSE;
    }
#endif

  server = gtk_editable_get_chars(GTK_EDITABLE(ctlRN.combo_entry_server),0,-1);
  port = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ctlRN.spinbutton_port));

  if((stm = gj_stream_new()) == NULL)
    {
      g_warning("gj_gtk_rn_connection: failed to send request, information query was NULL");
      return FALSE;
    }


  /* set struct info */
  gj_stream_set_to(stm, server);
  gj_stream_set_port(stm, port);
  gj_stream_set_connected_cb(stm,gj_gtk_rn_connected_response);

  if(gj_stream_build(stm) == FALSE)
    {
      g_warning("gj_gtk_rn_connection: failed to send request for authorisation, could not build stream.");
      return FALSE;
    }

  /* set controls state */
  gtk_widget_set_sensitive(ctlRN.vbox, FALSE);
  gtk_widget_set_sensitive(ctlRN.button_ok, FALSE);

  gj_stream_open(stm);
  
  /* clean up */
  if(server != NULL) g_free(server);

  /* make available to others */
  optRN.stm = stm;

  return TRUE;
}


void gj_gtk_rn_connected_response(stream stm, gint requestID)
{
  gchar *server = NULL;
  gchar *username = NULL;
  gchar *password = NULL;
  guint port = 5222;

  /* get information */
  server = gtk_editable_get_chars(GTK_EDITABLE(ctlRN.combo_entry_server),0,-1);
  port = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ctlRN.spinbutton_port));
  username = gtk_editable_get_chars(GTK_EDITABLE(ctlRN.entry_username),0,-1);
  password = gtk_editable_get_chars(GTK_EDITABLE(ctlRN.entry_password1),0,-1);

  /* set information */
  gj_me_set_connected_host(server);
  gj_me_set_connected_port(port);

  /* do request */
  gj_iq_request_register_user(server,
			      username,
			      password,
			      gj_gtk_rn_registered_response);

  /* clean up */
  if(server != NULL) g_free(server);
  if(username != NULL) g_free(username);
  if(password != NULL) g_free(password);
}

void gj_gtk_rn_registered_response(guint eventID)
{
  infoquery iq = NULL;
  infoquery iq_related = NULL;

  guint eventIDRelated = 0;

  xmlChar *type = NULL;
  xmlNodePtr node = NULL;

  gtk_widget_set_sensitive(ctlRN.vbox, TRUE);
  gtk_widget_set_sensitive(ctlRN.button_ok, TRUE);

  g_message("gj_gtk_rn_registered_response: response with eventID:%d",eventID);
#if 0
  if((iq = infoqueryFindByResponseID(eventID)) == NULL)
    {
      g_warning("gj_gtk_rn_registered_response: failed to find IQ based on eventID:%d",eventID);

      return;
    }
#endif
  
  if((iq = gj_iq_find_by_event_id(eventID)) == NULL)
    {
      gj_gtk_msg_new(TRUE,
		     GJ_GTK_MSG_BUTTON_CLOSE | GJ_GTK_MSG_ICON_EXCLAIMATION,
		     (MsgCb)NULL,
		     NULL,
		     _("Failed to Register New Account."),
		     _("Invalid response received."),
		     NULL);

       g_warning("gj_gtk_rn_registered_response: failed to find IQ based on eventID:%d",eventID);

      return;
    }

  eventIDRelated = gj_iq_get_related_id(iq);
  if((iq_related = gj_iq_find_by_event_id(eventIDRelated)) == NULL)
    {
      gj_gtk_msg_new(TRUE,
		     GJ_GTK_MSG_BUTTON_CLOSE | GJ_GTK_MSG_ICON_EXCLAIMATION,
		     (MsgCb)NULL,
		     NULL,
		     _("Failed to Register New Account."),
		     _("Invalid response received."),
		     NULL);

      g_warning("gj_gtk_rn_registered_response: failed to find IQ based on eventIDRelated:%d",eventIDRelated);

      return;
    }

  /* check for errors? */
  node = gj_iq_get_xml(iq_related);
  type = gj_parser_find_by_attr_and_ret(node, (guchar*)"type");

  if(type != NULL && xmlStrcmp(type, (guchar*)"error") == 0)
    {
      gchar *detail = NULL;

      xmlChar *errorCode = NULL;
      xmlChar *errorMessage = NULL;

      if((node = gj_parser_find_by_node(node, (guchar*)"error")) == NULL)
	{
	  g_warning("gj_gtk_rn_registered_response: failed to find node 'error'");
	  gj_iq_free_by_event_id(eventID);
	  return;
	}

      if((errorCode = gj_parser_find_by_attr_and_ret(node, (guchar*)"code")) != NULL)
	  detail = g_strdup_printf("%s:%s", _("Error"), errorCode);

      if((errorMessage = gj_parser_find_by_node_and_ret(node, (guchar*)"error")) != NULL) 
	{
	  if(detail != NULL)
	    detail = g_strdup_printf("%s - %s",detail,errorMessage);
	}

      gj_gtk_msg_new(TRUE,
		     GJ_GTK_MSG_BUTTON_CLOSE | GJ_GTK_MSG_ICON_EXCLAIMATION,
		     (MsgCb)NULL,
		     NULL,
		     _("Failed to Register New Account."),
		     "%s %s - %s.",
		     _("Error"),
		     errorCode?(gchar*)errorCode:"",
		     errorMessage?(gchar*)errorMessage:"",
		     NULL);

      g_warning("gj_gtk_rn_registered_response: IQ type is error, registration failed for eventID:%d",eventID);
      
      /* clean up */
      g_free(detail);
      gj_iq_free_all_related_by_event_id(eventID);
      
      /* find stream and close */
      if(optRN.stm != NULL)
	{
	  gj_stream_free(optRN.stm);
	  optRN.stm = NULL;
	}
      
      return;
    }

  /* ok */
  gj_gtk_msg_new(TRUE,
		 GJ_GTK_MSG_BUTTON_CLOSE | GJ_GTK_MSG_ICON_INFORMATION,
		 (MsgCb)NULL,
		 NULL,
		 _("Registration Success."),
		 _("No Errors"),
		 NULL);

  /* clean up */
  gj_iq_free_all_related_by_event_id(eventID);

  if(optRN.stm != NULL)
    {
      /* find stream and close */
      gj_stream_free(optRN.stm);
      optRN.stm = NULL;
    }

  /* should we save the settings to the connection settings ?? */
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlRN.checkbutton_use_configuration)) == TRUE)
    gj_gtk_rn_save_settings();
}

/*
 * actual gui functions
 */
void on_entry_rn_password1_changed(GtkEditable *editable, gpointer user_data)
{
    gj_gtk_rn_passwords_changed();
}

void on_entry_rn_password2_changed(GtkEditable *editable, gpointer user_data)
{
    gj_gtk_rn_passwords_changed();
}

void on_button_rn_ok_clicked(GtkButton *button, gpointer user_data)
{
  if(gj_gtk_rn_check() == FALSE)
    return;

  gj_gtk_rn_connection();
}

void on_button_rn_cancel_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_rn_unload();
}

gboolean on_register_new_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  /* !!!!!!! needs handling....!!!! */
  gj_gtk_rn_unload();
  return TRUE;
}

#ifdef __cplusplus
}
#endif
