/*
 * $RCSfile: gj_gtk_support.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/07/31 16:43:55 $
 *
 * $Revision: 1.8 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>

#include <glade/glade.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <glib.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "gj_support.h"

#include "gj_gtk_support.h"


#ifdef WIN32

#include <windows.h>
#include <shellapi.h>

/*
 * Windows code
 */
const char *gj_gtk_support_open_url_translate_error(int err);

gboolean gj_gtk_support_open_url(const gchar *url)
{
  HINSTANCE err;

  if(url == NULL)
    {
      g_warning("gj_gtk_support_open_url: url was NULL");
      return FALSE;
    }

  g_message("gj_gtk_support_open_url: starting browser for url:'%s'...", url);
    
  err = ShellExecute((HWND)NULL, /* parent window */
		     "open",     /* verb */
		     url,        /* file */
		     NULL,       /* parameters */
		     NULL,       /* path */
		     SW_SHOWNORMAL);
    
  if((gint)err <= 32)
    {
      g_warning("gj_gtk_support_open_url: failed to execute browser, error:%d->'%s', ", 
		(gint)err, gj_gtk_support_open_url_translate_error((gint)err));
      return FALSE;
    }

  return TRUE;
}

const char *gj_gtk_support_open_url_translate_error(int err)
{
  switch(err)
    {
    case 0: return "The operating system is out of memory or resources.";
    case ERROR_FILE_NOT_FOUND: return "The specified file was not found.";
    case ERROR_PATH_NOT_FOUND: return "The specified path was not found."; 
    case ERROR_BAD_FORMAT: return "The .exe file is invalid (non-Microsoft Win32 .exe or error in .exe image).";
    case SE_ERR_ACCESSDENIED: return "The operating system denied access to the specified file."; 
    case SE_ERR_ASSOCINCOMPLETE: return "The file name association is incomplete or invalid."; 
    case SE_ERR_DDEBUSY: return "The Dynamic Data Exchange (DDE) transaction could not be completed because other DDE transactions were being processed.";
    case SE_ERR_DDEFAIL: return "The DDE transaction failed.";
    case SE_ERR_DDETIMEOUT: return "The DDE transaction could not be completed because the request timed out.";
    case SE_ERR_DLLNOTFOUND: return "The specified dynamic-link library (DLL) was not found.";
      /*     case SE_ERR_FNF: return "The specified file was not found."; */
    case SE_ERR_NOASSOC: return "There is no application associated with the given file name extension. This error will also be returned if you attempt to print a file that is not printable.";
    case SE_ERR_OOM: return "There was not enough memory to complete the operation.";
      /*     case SE_ERR_PNF: return "The specified path was not found."; */
    case SE_ERR_SHARE: return "A sharing violation occurred.";
    }

  return "unknown error";
}

gboolean gj_gtk_support_sound_play(const gchar *filename)
{
  if(filename == NULL)
    {
      g_warning("gj_gtk_support_sound_play: filename was NULL");
      return FALSE;
    }

  g_message("gj_gtj_support_sound_play: this feature is not yet supported under Windows");
  return TRUE;
}


#else

#ifdef HAVE_GNOME
#include <libgnome/libgnome.h>
#endif

/*
 * Linux code
 */
gboolean gj_gtk_support_open_url(const gchar *url)
{
  GError *error = NULL;
  /*     gchar *full_url = NULL; */

  if(url == NULL)
    {
      g_warning("gj_gtk_support_open_url: url was NULL");
      return FALSE;
    }

  g_message("gj_gtk_support_open_url: starting browser for url:'%s'...", url);

  /*     full_url = g_strconcat("\"", url, "\"", NULL); */
    
  if(gnome_url_show(url, &error) == FALSE)
    g_warning("gj_gtk_support_open_url: gnome_url_show failed.");
    
  /* look at error */
  if(error != NULL)
    {
      g_warning("gj_gtk_support_open_url: error domain:'%s', code:%d, message:'%s'", 
		g_quark_to_string(error->domain), error->code, error->message);

      g_error_free(error);
      return FALSE;
    }

  return TRUE;    
}

gboolean gj_gtk_support_sound_play(const gchar *filename)
{
  if(filename == NULL)
    {
      g_warning("gj_gtk_support_sound_play: filename was NULL");
      return FALSE;
    }

  g_message("gj_gtk_support_sound_play: playing sound file:'%s'", filename);
  gnome_sound_play(filename);
  return TRUE;
}

#endif


GtkWidget *gj_gtk_support_image_new_from_file(const gchar *filename)
{
  GtkWidget *image = NULL;

  if(filename == NULL || g_utf8_strlen(filename, -1) < 1)
    {
      g_warning("gj_gtk_support_image_new_from_file: filename was NULL");
      return NULL;
    }
  else
    g_message("gj_gtk_support_image_new_from_file: opening file:'%s'", filename);

  image = gtk_image_new_from_file(filename);

  if(image == NULL)
    {
      g_warning("gj_gtk_support_image_new_from_file: image was NULL");
      return NULL;
    }

  return image;
}

GdkPixbuf *gj_gtk_support_pixbuf_new_from_file(const gchar *filename)
{
  GdkPixbuf *pb = NULL;
  GError *error = NULL;

  if(filename == NULL || g_utf8_strlen(filename, -1) < 1)
    {
      g_warning("gj_gtk_support_pixbuf_new_from_file: filename was NULL");
      return NULL;
    }
  else
    g_message("gj_gtk_support_pixbuf_new_from_file: opening file:'%s'", filename);

  pb = gdk_pixbuf_new_from_file(filename, &error);

  if(pb == NULL)
    {
      g_warning("gj_gtk_support_pixbuf_new_from_file: error loading file:'%s'", filename);
	
      /* look at error */
      if(error != NULL)
	{
	  g_warning("gj_gtk_support_pixbuf_new_from_file: error domain:'%s', code:%d, message:'%s'", 
		    g_quark_to_string(error->domain), error->code, error->message);

	  g_error_free(error);
	}
	
      return NULL;
    }

  /* NOTE: this has a refcount of 1 it needs to be unref'd */
  return pb;
}

GdkPixbuf *gj_gtk_support_pixbuf_new_from_stock(const gchar *stock_item)
{
  GdkPixbuf *pb = NULL;
  GtkWidget *image = NULL;

  if(stock_item == NULL || g_utf8_strlen(stock_item, -1) < 1)
    {
      g_warning("gj_gtk_support_pixbuf_new_from_stock: stock_item was NULL");
      return NULL;
    }
  else
    g_message("gj_gtk_support_pixbuf_new_from_stock: opening stock item:'%s'", stock_item);

  /* create image to have a widget to render the icon */
  image = gtk_image_new();

  /* get pixbuf */
  pb = gtk_widget_render_icon(GTK_WIDGET(image), stock_item, GTK_ICON_SIZE_SMALL_TOOLBAR, NULL);

  if(pb == NULL)
    {
      g_warning("gj_gtk_support_pixbuf_new_from_stock: could not get pixbuf from stock item:'%s'", stock_item);
      return NULL;
    }

  /* clean up */
  if(image != NULL) g_object_unref(G_OBJECT(image));

  return pb;
}

GdkPixbufAnimation *gj_gtk_support_animation_new_from_file(const gchar *filename)
{
  GdkPixbufAnimation *pba = NULL;
  GError *error = NULL;

  if(filename == NULL || g_utf8_strlen(filename, -1) < 1)
    {
      g_warning("gj_gtk_support_animation_new_from_file: filename was NULL");
      return NULL;
    }
  else
    g_message("gj_gtk_support_animation_new_from_file: opening file:'%s'", filename);

  pba = gdk_pixbuf_animation_new_from_file(filename, &error);

  if(pba == NULL)
    {
      g_warning("gj_gtk_support_animation_new_from_file: error loading file:'%s'", filename);
	
      /* look at error */
      if(error != NULL)
	{
	  g_warning("gj_gtk_support_animation_new_from_file: error domain:'%s', code:%d, message:'%s'", 
		    g_quark_to_string(error->domain), error->code, error->message);

	  g_error_free(error);
	}
	
      return NULL;
    }

  /* NOTE: this has a refcount of 1 it needs to be unref'd */
  return pba;
}

gboolean gj_gtk_support_find_in_str(gchar *needle, gchar *haystack)
{
  gchar *result = NULL;

  if(needle == NULL)
    {
      g_warning("gj_gtk_support_find_in_str: needle is NULL");
      return FALSE;
    }

  if(haystack == NULL)
    {
      g_warning("gj_gtk_support_find_in_str: haystack is NULL");
      return FALSE;
    }

  if((result = g_strrstr(haystack, needle)) == NULL)
    return FALSE;
    
  return TRUE;
}

gchar *gj_gtk_support_filter_number(gchar *original_number)
{
  gchar *number_filtered = NULL;
  gchar *number = NULL;
  gchar **vnumber = NULL;

  /* checks */
  if(original_number == NULL)
    {
      g_warning("gj_gtk_support_filter_number: original_number is NULL");
      return NULL;
    }  

  /* remove all except numbers */
  number = g_strdup(original_number);
  number = g_strcanon(number, "0123456789", 0x20);
  vnumber = g_strsplit(number, " ", -1);
    
  if(vnumber != NULL)
    {
      number_filtered = g_strjoinv(NULL, vnumber);
      g_strfreev(vnumber);
      vnumber = NULL;
    }

  /* clean up */
  if(number != NULL) g_free(number);

  return number_filtered;
}


#ifdef __cplusplus
}
#endif
