/* 
 * $RCSfile: gj_presence.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/08/17 18:51:17 $
 * 
 * $Revision: 1.11 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <stdlib.h>
#include <string.h>

#include <time.h>

#include <glib.h>

#include <libxml/tree.h>

#include "gj_presence.h"
#include "gj_parser.h"
#include "gj_support.h"


struct t_presence
{
  presenceType type;
  presenceShow show;

  gchar *status;
  gint8 priority; /* -128 to +127 */
  
  gint requestID;

  gchar *to_jid;
  gchar *to_resource;
  gchar *from_jid;
  gchar *from_resource;
  
  gchar *error_code;
  gchar *error_reason;

  time_t time;
};


/* to details, i.e. my state */
presenceToCallback gj_presence_get_cb_to();

/* from details, i.e. my contacts state(s) */
presenceFromCallback gj_presence_get_cb_from();

/* function */
void gj_presence_destroy(presence pres);

gboolean gj_presence_list_add(presence pres);
gboolean gj_presence_list_del(presence pres);


GList *presence_list = NULL;

presenceFromCallback presence_cb_from;
presenceToCallback presence_cb_to;   /* may not get used */


/*
 * code
 */
gboolean gj_presence_init()
{
  return TRUE;
}

gboolean gj_presence_term()
{
  gint index = 0;

  /* list */
  if(presence_list == NULL)
    return TRUE;

  /* free all items on the list */
  for(index=0;index<g_list_length(presence_list);index++)
    {
      presence pres = NULL;
      
      if((pres = (presence) g_list_nth_data(presence_list,index)) != NULL)
	gj_presence_free(pres);
    }

  g_list_free(presence_list);
  presence_list = NULL;

  return TRUE;
}

/* list functions */
gboolean gj_presence_list_add(presence pres)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return FALSE;
    }

  presence_list = g_list_append(presence_list,pres);

  return TRUE;
}

gboolean gj_presence_list_del(presence pres)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return FALSE;
    }

  presence_list = g_list_remove(presence_list,pres);
  return TRUE;
}

presence gj_presence_new(gint requestID, presenceType type, presenceShow show)
{
  presence pres = NULL;

  if(requestID < 1)
    {
      g_warning("%s: request id:%d was < 1", __FUNCTION__, requestID);
      return NULL;
    }
  
  if((pres = g_new(struct t_presence,1)) != NULL)
    {
      pres->type = type;
      pres->show = show;

      pres->requestID = requestID;

      pres->to_jid = NULL;
      pres->to_resource = NULL;

      pres->from_jid = NULL;
      pres->from_resource = NULL;
      
      pres->status = NULL;
      pres->priority = 0;

      pres->error_code = NULL;
      pres->error_reason = NULL;

      gj_presence_list_add(pres);
    }
  
  return pres;  
}

void gj_presence_free(presence pres)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return;
    }

  /* free mem */
  gj_presence_list_del(pres);  

  /* destroy members */
  gj_presence_destroy(pres);

  /* free structure */
  g_free(pres);
}

void gj_presence_destroy(presence pres)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return;
    }

  pres->type = presenceTypeEnd;
  pres->show = presenceShowEnd;

  pres->requestID = 0;

  if(pres->status != NULL)
    {
      g_free(pres->status);
      pres->status = NULL;
    }

  pres->priority = 0;

  if(pres->to_jid != NULL)
    {
      g_free(pres->to_jid);
      pres->to_jid = NULL;
    }

  if(pres->to_resource != NULL)
    {
      g_free(pres->to_resource);
      pres->to_resource = NULL;
    }

  if(pres->from_jid != NULL)
    {
      g_free(pres->from_jid);
      pres->from_jid = NULL;
    }

  if(pres->from_resource != NULL)
    {
      g_free(pres->from_resource);
      pres->from_resource = NULL;
    }  

  if(pres->error_code != NULL)
    {
      g_free(pres->error_code);
      pres->error_code = NULL;
    } 

  if(pres->error_reason != NULL)
    {
      g_free(pres->error_reason);
      pres->error_reason = NULL;
    } 
}

presence gj_presence_copy(presence pres)
{
  presence pres_copy = NULL;

  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return NULL;
    }

  if((pres_copy = gj_presence_new(gj_support_get_unique_id(), pres->type, pres->show)) == NULL)
    {
      g_warning("%s: could not create new presence", __FUNCTION__);
      return NULL;
    }

  /* 
   * not copying requestID because this is supposed to be unique 
   */

  if(pres->status != NULL)
    pres_copy->status = g_strdup(pres->status);
  
  pres_copy->priority = pres->priority;
  
  if(pres->to_jid != NULL)
    pres_copy->to_jid = g_strdup(pres->to_jid);
  
  if(pres->to_resource != NULL)
    pres_copy->to_resource = g_strdup(pres->to_resource);
  
  if(pres->from_jid != NULL)
    pres_copy->from_jid = g_strdup(pres->from_jid);
  
  if(pres->from_resource != NULL)
    pres_copy->from_resource = g_strdup(pres->from_resource);
  
  if(pres->error_code != NULL)
    pres_copy->error_code = g_strdup(pres->error_code);
  
  if(pres->error_reason != NULL)
    pres_copy->error_reason = g_strdup(pres->error_reason);
  
  return pres_copy;
}

/* find functions */
presence gj_presence_find_by_to_jid(gchar *jid)
{
  gint index = 0;

  if(jid == NULL)
    {
      g_warning("%s: jid was NULL", __FUNCTION__);
      return NULL;
    }

  for(index=0;index<g_list_length(presence_list);index++)
    {
      presence pres  = g_list_nth_data(presence_list, index);

      if(pres == NULL) continue;
      if(strcmp(pres->to_jid,jid) == 0) return pres;
    }

  return NULL;
}

/*
 * member functions 
 */

/* sets */
gboolean gj_presence_set_to_jid(presence pres, gchar *jid)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return FALSE;
    }

  if(jid == NULL)
    {
      g_warning("%s: jid was NULL", __FUNCTION__);
      return FALSE;
    }

  if(pres->to_jid != NULL)
    g_free(pres->to_jid);

  pres->to_jid = g_strdup(jid);
  return TRUE;
}

gboolean gj_presence_set_type(presence pres, presenceType type)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return FALSE;
    }

  if(type < 1 || type > presenceTypeEnd)
    {
      g_warning("%s: type was out of range", __FUNCTION__);
      return FALSE;
    }

  pres->type = type;
  return TRUE;
}

gboolean gj_presence_set_show(presence pres, presenceShow show)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return FALSE;
    }

  if(show < 1 || show > presenceShowEnd)
    {
      g_warning("%s: type was out of range", __FUNCTION__);
      return FALSE;
    }

  pres->show = show;
  return TRUE;
}

gboolean gj_presence_set_status(presence pres, gchar *status)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return FALSE;
    }

  if(status == NULL)
    {
      g_warning("%s: status was NULL", __FUNCTION__);
      return FALSE;
    }

  if(pres->status != NULL)
    g_free(pres->status);

  pres->status = g_strdup(status);
  return TRUE;
}

gboolean gj_presence_set_priority(presence pres, gint8 priority)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return FALSE;
    }

  if(priority < 0)
    {
      g_warning("%s: priority:%d was < 0", __FUNCTION__, priority);
      return FALSE;
    }

  pres->priority = priority;
  return TRUE;
}

gboolean gj_presence_set_to_resource(presence pres, gchar *resource)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return FALSE;
    }

  if(resource == NULL)
    {
      g_warning("%s: resource was NULL", __FUNCTION__);
      return FALSE;
    }

  if(pres->to_resource != NULL)
    g_free(pres->to_resource);

  pres->to_resource = g_strdup(resource);
  return TRUE;
}

gboolean gj_presence_set_from_jid(presence pres, gchar *jid)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return FALSE;
    }

  if(jid == NULL)
    {
      g_warning("%s: jid was NULL", __FUNCTION__);
      return FALSE;
    }

  if(pres->from_jid != NULL)
    g_free(pres->from_jid);

  pres->from_jid = g_strdup(jid);
  return TRUE;
}

gboolean gj_presence_set_from_resource(presence pres, gchar *resource)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return FALSE;
    }

  if(resource == NULL)
    {
      g_warning("%s: resource was NULL", __FUNCTION__);
      return FALSE;
    }

  if(pres->from_resource != NULL)
    g_free(pres->from_resource);

  pres->from_resource = g_strdup(resource);
  return TRUE;
}

gboolean gj_presence_set_error_code(presence pres, gchar *error_code)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return FALSE;
    }

  if(error_code == NULL)
    {
      g_warning("%s: error code was NULL", __FUNCTION__);
      return FALSE;
    }

  if(pres->error_code != NULL)
    g_free(pres->error_code);

  pres->error_code = g_strdup(error_code);
  return TRUE;
}

gboolean gj_presence_set_error_reason(presence pres, gchar *error_reason)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return FALSE;
    }

  if(error_reason == NULL)
    {
      g_warning("%s: error reason was NULL", __FUNCTION__);
      return FALSE;
    }

  if(pres->error_reason != NULL)
    g_free(pres->error_reason);

  pres->error_reason = g_strdup(error_reason);
  return TRUE;
}

/* gets */
presenceType gj_presence_get_type(presence pres)
{ 
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return -1;
    }  

  return pres->type;
}

presenceShow gj_presence_get_show(presence pres)
{ 
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return -1;
    }  

  return pres->show;
}

gchar *gj_presence_get_status(presence pres)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return NULL;
    }  

  return pres->status;
}

gint8 gj_presence_get_priority(presence pres)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return 0;
    }  

  return pres->priority;
}

gint gj_presence_get_request_id(presence pres)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return -1;
    }  

  return pres->requestID;
}

gchar *gj_presence_get_to_jid(presence pres)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return NULL;
    }  

  return pres->to_jid;
}

gchar *gj_presence_get_to_resource(presence pres)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return NULL;
    }  

  return pres->to_resource;
}

gchar *gj_presence_get_from_jid(presence pres)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return NULL;
    }  

  return pres->from_jid;
}

gchar *gj_presence_get_from_resource(presence pres)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return NULL;
    }  

  return pres->from_resource;
}

gchar *gj_presence_get_error_code(presence pres)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return NULL;
    }  

  return pres->error_code;
}

gchar *gj_presence_get_error_reason(presence pres)
{
  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return FALSE;
    }  

  return pres->error_reason;
}

/*
 * FROM presence
 */
gboolean gj_presence_set_cb_from(presenceFromCallback cb)
{
  if(cb == NULL)
    {
      g_warning("%s: callback was NULL", __FUNCTION__);
      return FALSE;
    }

  presence_cb_from = cb;
  
  return TRUE;
}

presenceFromCallback gj_presence_get_cb_from()
{
  return presence_cb_from;
}

/*
 * TO presence
 */
gboolean gj_presence_set_cb_to(presenceToCallback proc)
{
  if(proc == NULL)
    {
      g_warning("%s: callback was NULL", __FUNCTION__);
      return FALSE;
    }

  presence_cb_to = proc;
  
  return TRUE;
}

presenceToCallback gj_presence_get_cb_to()
{
  return presence_cb_to;
}


/*
 * send/receive functions
 */
gboolean gj_presence_send(presenceType type, presenceShow show, gchar *status, gint8 priority)
{
  xmlNodePtr node = NULL;
  xmlNodePtr sub = NULL;
  xmlNsPtr ns = NULL;

  gchar *id = NULL;

  if(show < 1)
    return FALSE;

  id = g_strdup_printf("%d", gj_support_get_unique_id());
  
  node = xmlNewNode(NULL, (guchar*)"presence");

  xmlSetProp(node, (guchar*)"id", (guchar*)id);

  if(type >= 1)
    {
      switch(type)
	{
	case presenceTypeUnAvailable:
	  xmlSetProp(node, (guchar*)"type", (guchar*)"unavailable");
	  break;
	case presenceTypeAvailable:
	  xmlSetProp(node, (guchar*)"type", (guchar*)"available");
	  break;
	case presenceTypeProbe:
	  xmlSetProp(node, (guchar*)"type", (guchar*)"probe");
	  break;
	case presenceTypeSubscribe:
	  xmlSetProp(node, (guchar*)"type", (guchar*)"subscribe");
	  break;
	case presenceTypeUnSubscribe:
	  xmlSetProp(node, (guchar*)"type", (guchar*)"unsubscribe");
	  break;
	case presenceTypeSubscribed:
	  xmlSetProp(node, (guchar*)"type", (guchar*)"subscribed");
	  break;
	case presenceTypeUnSubscribed:
	  xmlSetProp(node, (guchar*)"type", (guchar*)"unsubscribed");
	  break;
	  
	default:
	case presenceTypeEnd:
	  break;	  
	}
    }
  
  /* add show */ 
  switch(show)
    {
    case presenceShowAway:
      sub = xmlNewChild(node, ns, (guchar*)"show", (guchar*)"away");
      break;
    case presenceShowChat:
      sub = xmlNewChild(node, ns, (guchar*)"show", (guchar*)"chat");
      break;
    case presenceShowDND:
      sub = xmlNewChild(node, ns, (guchar*)"show", (guchar*)"dnd");
      break;
    case presenceShowXA:
      sub = xmlNewChild(node, ns, (guchar*)"show", (guchar*)"xa");
      break;
    case presenceShowEnd:
      break;

    default:
      sub = xmlNewChild(node, ns, (guchar*)"show", (guchar*)"normal");
      break;
    }

  /* add status */
  if(status != NULL)
    {
      gchar *formatted = NULL;
      formatted = g_markup_escape_text(status, -1);
      sub = xmlNewChild(node, ns, (guchar*)"status", (guchar*)formatted);
      if(formatted != NULL) g_free(formatted);
    }

  if(TRUE)
    {
      gchar *formatted = NULL;
      formatted = g_strdup_printf("%d", priority);
      sub = xmlNewChild(node, ns, (guchar*)"priority", (guchar*)formatted);
      if(formatted != NULL) g_free(formatted);
    }
  
  gj_parser_push_output(node);

  /* clean up */
  if(id != NULL) g_free(id);

  return TRUE;
}

gboolean gj_presence_send_to(gchar *jid, presenceType type, presenceShow show, gchar *status, gint8 priority)
{
  xmlNodePtr node = NULL;
  xmlNodePtr sub = NULL;
  xmlNsPtr ns = NULL;

  gchar *id = NULL;

  if(show < 1)
    return FALSE;

  if(jid == NULL)
    {
      g_warning("%s: jid was NULL", __FUNCTION__);
      return FALSE;
    }

  id = g_strdup_printf("%d", gj_support_get_unique_id());
  
  node = xmlNewNode(NULL, (guchar*)"presence");

  xmlSetProp(node, (guchar*)"id", (guchar*)id);
  xmlSetProp(node, (guchar*)"to", (guchar*)jid);

  if(type >= 1)
    {
      switch(type)
	{
	case presenceTypeUnAvailable:
	  xmlSetProp(node, (guchar*)"type", (guchar*)"unavailable");
	  break;
	case presenceTypeAvailable:
	  xmlSetProp(node, (guchar*)"type", (guchar*)"available");
	  break;
	case presenceTypeProbe:
	  xmlSetProp(node, (guchar*)"type", (guchar*)"probe");
	  break;
	case presenceTypeSubscribe:
	  xmlSetProp(node, (guchar*)"type", (guchar*)"subscribe");
	  break;
	case presenceTypeUnSubscribe:
	  xmlSetProp(node, (guchar*)"type", (guchar*)"unsubscribe");
	  break;
	case presenceTypeSubscribed:
	  xmlSetProp(node, (guchar*)"type", (guchar*)"subscribed");
	  break;
	case presenceTypeUnSubscribed:
	  xmlSetProp(node, (guchar*)"type", (guchar*)"unsubscribed");
	  break;
	  
	default:
	case presenceTypeEnd:
	  break;	  
	}
    }
  
  /* add show */ 
  switch(show)
    {
    case presenceShowAway:
      sub = xmlNewChild(node, ns, (guchar*)"show", (guchar*)"away");
      break;
    case presenceShowChat:
      sub = xmlNewChild(node, ns, (guchar*)"show", (guchar*)"chat");
      break;
    case presenceShowDND:
      sub = xmlNewChild(node, ns, (guchar*)"show", (guchar*)"dnd");
      break;
    case presenceShowXA:
      sub = xmlNewChild(node, ns, (guchar*)"show", (guchar*)"xa");
      break;
    case presenceShowEnd:
      break;

    default:
      sub = xmlNewChild(node, ns, (guchar*)"show", (guchar*)"normal");
      break;
    }

  /* add status */
  if(status != NULL)
    {
      gchar *formatted = NULL;
      formatted = g_markup_escape_text(status, -1);
      sub = xmlNewChild(node, ns, (guchar*)"status", (guchar*)formatted);
      if(formatted != NULL) g_free(formatted);
    }

  if(TRUE)
    {
      gchar *formatted = NULL;
      formatted = g_strdup_printf("%d", priority);
      sub = xmlNewChild(node, ns, (guchar*)"priority", (guchar*)formatted);
      if(formatted != NULL) g_free(formatted);
    }
  
  gj_parser_push_output(node);

  /* clean up */
  if(id != NULL) g_free(id);

  return TRUE;
}

void gj_presence_receive(connection c, xmlNodePtr node)
{
  gint requestID = gj_support_get_unique_id();

  xmlChar *tmp = NULL;

  xmlChar *stype = NULL;
  xmlChar *sshow = NULL;
  xmlChar *spriority = NULL;

  presence pres = NULL;
  presenceFromCallback callback = NULL;

  /* checks */
  if(node == NULL)
    {
      g_warning("%s: xml node was NULL", __FUNCTION__);
      return;
    }

  /* get info */
  if((callback = gj_presence_get_cb_from()) == NULL)
    {
      g_warning("%s: callback was NULL", __FUNCTION__);
      return;
    }

  if((pres = gj_presence_new(requestID, presenceTypeAvailable, presenceShowNormal)) == NULL)
    {
      g_warning("%s: could not create new presence", __FUNCTION__);
      return;
    }

  if((stype = gj_parser_find_by_attr_and_ret(node, (guchar*)"type")) != NULL)
    {
      if(xmlStrcmp(stype, (guchar*)"available") == 0)
	gj_presence_set_type(pres, presenceTypeAvailable);
      else if(xmlStrcmp(stype, (guchar*)"unavailable") == 0)
	gj_presence_set_type(pres, presenceTypeUnAvailable);
      else if(xmlStrcmp(stype, (guchar*)"probe") == 0)
	gj_presence_set_type(pres, presenceTypeProbe);
      else if(xmlStrcmp(stype, (guchar*)"subscribed") == 0)
	gj_presence_set_type(pres, presenceTypeSubscribed);
      else if(xmlStrcmp(stype, (guchar*)"unsubscribed") == 0)
	gj_presence_set_type(pres, presenceTypeUnSubscribed);
      else if(xmlStrcmp(stype, (guchar*)"subscribe") == 0)
	gj_presence_set_type(pres, presenceTypeSubscribe);
      else if(xmlStrcmp(stype, (guchar*)"unsubscribe") == 0)
	gj_presence_set_type(pres, presenceTypeUnSubscribe);
      else if(xmlStrcmp(stype, (guchar*)"error") == 0)
	gj_presence_set_type(pres, presenceTypeError);
    }

  if((sshow = gj_parser_find_by_node_and_ret(node, (guchar*)"show")) != NULL)
    {
      if(xmlStrcmp(sshow, (guchar*)"normal") == 0)
	gj_presence_set_show(pres, presenceShowNormal);
      else if(xmlStrcmp(sshow, (guchar*)"away") == 0)
	gj_presence_set_show(pres, presenceShowAway);
      else if(xmlStrcmp(sshow, (guchar*)"chat") == 0)
	gj_presence_set_show(pres, presenceShowChat);
      else if(xmlStrcmp(sshow, (guchar*)"dnd") == 0)
	gj_presence_set_show(pres, presenceShowDND);
      else if(xmlStrcmp(sshow, (guchar*)"xa") == 0)
	gj_presence_set_show(pres, presenceShowXA);
    }
  
  if((spriority = gj_parser_find_by_node_and_ret(node, (guchar*)"priority")) != NULL)
    {
      if(xmlStrlen(spriority) > 0) 
	gj_presence_set_priority(pres, atoi((gchar*)spriority));
    }
 
  if((tmp = gj_parser_find_by_attr_and_ret(node, (guchar*)"to")) != NULL && xmlStrlen(tmp) > 0)
    {
      gchar *to = g_strdup_printf("%s",tmp);
      gchar **vTo = g_strsplit(to,"/",2);

      if(vTo != NULL)
	{
	  gj_presence_set_to_jid(pres, to);

	  if(vTo[1] != NULL)
	    gj_presence_set_to_resource(pres, vTo[1]);

	  g_strfreev(vTo);
	}

      if(to != NULL) g_free(to);
    }

  if((tmp = gj_parser_find_by_attr_and_ret(node, (guchar*)"from")) != NULL && xmlStrlen(tmp) > 0)
    {
      gchar *from = g_strdup_printf("%s",tmp);
      gchar **vFrom = g_strsplit(from,"/",2);

      if(vFrom != NULL)
	{
	  gj_presence_set_from_jid(pres, (gchar*)tmp);

	  if(vFrom[1] != NULL)
	    gj_presence_set_from_resource(pres, vFrom[1]);

	  g_strfreev(vFrom);
	}

      if(from != NULL) g_free(from);
    }

  if((tmp = gj_parser_find_by_node_and_ret(node, (guchar*)"status")) != NULL)
    gj_presence_set_status(pres, (gchar*)tmp);
 
  /* if error deal with it */
  if(gj_presence_get_type(pres) == presenceTypeError)
    {
      /* get error information */
      xmlChar *code = NULL;
      const xmlChar *reason = NULL;
      xmlNodePtr node_error = NULL;

      if((node_error = gj_parser_find_by_node(node, (guchar*)"error")) != NULL)
	{
	  if((code = gj_parser_find_by_attr_and_ret(node_error, (guchar*)"code")) != NULL && 
	     xmlStrlen(code) > 0)
	    gj_presence_set_error_code(pres, (gchar*)code);
	  
	  if((reason = gj_parser_get_content(node_error)) != NULL && xmlStrlen(reason) > 0)
	    gj_presence_set_error_reason(pres, (gchar*)reason);
	}
    }

  /* call callback function */
  (callback)(pres);
}



#ifdef __cplusplus
}
#endif
