/* 
 * $RCSfile: gj_roster.h,v $
 *
 * $Author: ginxd $
 * $Date: 2003/08/19 00:52:08 $
 * 
 * $Revision: 1.13 $
 *
 */

#ifndef __gj_roster_h
#define __gj_roster_h

#include <glib.h>

#include "gj_presence.h"
#include "gj_message.h"
/*#include "gj_event.h"*/
#include "gj_iq.h"

/* 
 * Rosters 
 */
typedef enum rosterItemSubscription {
  rosterItemSubscriptionNone = 1,
  rosterItemSubscriptionTo,
  rosterItemSubscriptionFrom,
  rosterItemSubscriptionBoth,
  
  rosterItemSubscriptionEnd
} rosterItemSubscription;

typedef enum rosterItemType {
  rosterItemTypeNone = 1,
  rosterItemTypeContact,
  rosterItemTypeAgent,
  rosterItemTypeGroupChat, /* used for group chats */
  
  rosterItemTypeEnd
} rosterItemType;

typedef struct t_rosterItem *rosterItem;

typedef void (*rosterCallback)(GList *roster);

/* list functions */
gboolean gj_roster_init();
gboolean gj_roster_term();

/* list functions */
GList *gj_roster_get_list();

GList *gj_roster_get_all_groups();
GList *gj_roster_get_all_items_by_group(gchar *group);
GList *gj_roster_get_all_items_by_agent(gchar *agent);
GList *gj_roster_get_all_agents_by_jid();
GList *gj_roster_get_all_agents_by_jid_without_resource();
GList *gj_roster_get_all_agents_by_name();
gboolean gj_roster_is_agent_registered(gchar *agent);

/* item functions */
rosterItem gj_roster_item_new(gchar *jid);
rosterItem gj_roster_item_new_unknown_by_message(message msg); /* should no longer be used */
rosterItem gj_roster_item_new_unknown_by_jid(gchar *jid);
void gj_roster_item_free(rosterItem ri);

/* find functions */
rosterItem gj_roster_item_find_by_id(gint id);
rosterItem gj_roster_item_find_by_name(gchar *name);
rosterItem gj_roster_item_find_by_jid(gchar *jid);
rosterItem gj_roster_item_find_by_jid_with_resource(gchar *jid);
rosterItem gj_roster_item_find_by_jid_without_resource(gchar *jid);
rosterItem gj_roster_item_find_by_jid_ignore_resource(gchar *jid); /* this drops the resource on the string provided 
								      AND the jid's for each roster item */

/* 
 * member functions 
 */

/* sets */
gboolean gj_roster_item_set_jid(rosterItem ri, gchar *jid);
gboolean gj_roster_item_set_name(rosterItem ri, gchar *name);
gboolean gj_roster_item_set_resource(rosterItem ri, gchar *resource);
gboolean gj_roster_item_set_groups(rosterItem ri, GList *groups);
gboolean gj_roster_item_set_agent(rosterItem ri, gchar *agent);
gboolean gj_roster_item_set_is_agent(rosterItem ri, gboolean IsAgent);
gboolean gj_roster_item_set_subscription(rosterItem ri, rosterItemSubscription subscription);
gboolean gj_roster_item_set_type(rosterItem ri, rosterItemType type);
gboolean gj_roster_item_set_presence(rosterItem ri, presence pres);

/* gets */
gint gj_roster_item_get_id(rosterItem ri);
gchar *gj_roster_item_get_jid(rosterItem ri);
gchar *gj_roster_item_get_jid_without_resource(rosterItem ri);
gchar *gj_roster_item_get_name(rosterItem ri);
gchar *gj_roster_item_get_resource(rosterItem ri);
GList *gj_roster_item_get_groups(rosterItem ri);
gchar *gj_roster_item_get_groups_as_string(rosterItem ri);
gchar *gj_roster_item_get_agent(rosterItem ri);
gboolean gj_roster_item_get_is_agent(rosterItem ri);
gboolean gj_roster_item_get_is_permanent(rosterItem ri);
gboolean gj_roster_item_get_is_another_resource(rosterItem ri);
gboolean gj_roster_item_get_is_set_for_removal(rosterItem ri);           
rosterItemSubscription gj_roster_item_get_subscription(rosterItem ri);
rosterItemType gj_roster_item_get_type(rosterItem ri);
presence gj_roster_item_get_presence(rosterItem ri);

/* item group functions */
gboolean gj_roster_item_groups_add(rosterItem ri, gchar *group);
gboolean gj_roster_item_groups_del(rosterItem ri, gchar *group);
gint gj_roster_item_groups_count(rosterItem ri);
gboolean gj_roster_item_groups_free(rosterItem ri);

/* inbound */
GList *gj_roster_parse(xmlNodePtr parent);
gboolean gj_roster_parse_set_request(infoquery iq);

/* debugging */
void gj_roster_print_all();
void gj_roster_item_print(rosterItem ri);

#endif

