/* 
 * $RCSfile: gj_stream.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/08/20 15:31:28 $
 * 
 * $Revision: 1.12 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>

#include <glib.h>

#include <libxml/tree.h>

#include "common.h"

#include "gj_stream.h"
#include "gj_parser.h"
#include "gj_connection.h"
#include "gj_support.h"

#include "gj_gtk_msg.h"

/* item functions */
void gj_stream_destroy(stream stm);

/*
 * member functions
 */

/* sets */
gboolean gj_stream_set_id(stream stm, gchar *id);

gboolean gj_stream_set_stream(stream stm, gchar *stream);
gboolean gj_stream_set_xml(stream stm, xmlDocPtr doc);

gboolean gj_stream_set_connection(stream stm, connection c);

void gj_stream_set_is_built(stream stm, gboolean is_built);

/* gets */
gboolean gj_stream_get_is_built(stream stm);

/* utils */
gchar *gj_stream_convert_doc_to_str(stream stm);

/* callbacks */
void gj_stream_connected_cb(connection c);
void gj_stream_disconnected_cb(connection c);
void gj_stream_error_cb(connection c, GTcpSocketConnectAsyncStatus status);

/* structs */
struct t_stream
{
  gint requestID;

  streamCallback cb_connected;
  streamCallback cb_disconnected;
  streamCallback cb_error;

  gboolean is_built;

  gchar *id;
  gchar *stream;
  gchar *to;

  guint unique_id;

  guint port;

  connection c;

  /*xmlDocPtr doc;*/
  xmlDocPtr xml;
};


GList *stream_list;

gboolean gj_stream_init()
{
  if(stream_list != NULL)
    gj_stream_term();

  /* no list to create, this is done on the first g_list_append */

  return TRUE;
}

gboolean gj_stream_term()
{  
  gint index = 0;

  /* list */
  if(stream_list == NULL)
    return TRUE;

  /* free all items on the list */
  for(index=0;index<g_list_length(stream_list);index++)
    {
      stream stm = NULL;
      
      if((stm = (stream) g_list_nth_data(stream_list,index)) != NULL)
	gj_stream_free(stm);
    }

  g_list_free(stream_list);
  stream_list = NULL;

  return TRUE;
}

stream gj_stream_new(gchar *host)
{
  stream stm = NULL;

  if(host == NULL)
    {
      g_warning("%s: host was NULL", __FUNCTION__);
      return NULL;
    }

  if(gj_stream_find_by_to(host) != NULL)
    {
      g_warning("%s: there is already a stream to:'%s'", __FUNCTION__, host);
      return NULL;
    }
  
  if((stm = (stream) g_new(struct t_stream,1)))
    {
      stm->to = g_strdup(host);

      stm->requestID = gj_support_get_unique_id(); /* get next */

      stm->xml = NULL;

      stm->cb_connected = NULL;
      stm->cb_disconnected = NULL;
      stm->cb_error = NULL;

      stm->is_built = FALSE;

      stm->id = NULL;
      stm->stream = NULL;

      stm->port = 5222;

      stm->c = NULL;

      stm->unique_id = gj_support_get_unique_id();

      stream_list = g_list_append(stream_list,stm);
    }
  
  return stm;  
}

gboolean gj_stream_free(stream stm)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return FALSE;
    }

  /* free mem */
  stream_list = g_list_remove(stream_list,stm);  
  
  /* destroy members */
  gj_stream_destroy(stm);

  /* free structure */
  g_free(stm);

  return TRUE;
}

void gj_stream_destroy(stream stm)
{
  if(stm == NULL)
    {
      g_warning("%s: failed to destroy, stm was NULL", __FUNCTION__);
      return;
    }

  stm->requestID = 0; 
  stm->is_built = FALSE;
  
  if(stm->xml != NULL)
    {
      xmlFreeDoc(stm->xml);
      stm->xml = NULL;
    }

  stm->cb_connected = NULL;
  stm->cb_disconnected = NULL;
  stm->cb_error = NULL;

  if(stm->id != NULL)
    {
      g_free(stm->id);
      stm->id = NULL;
    }

  if(stm->to != NULL)
    {
      g_free(stm->to);
      stm->to = NULL;
    }

  if(stm->stream != NULL)
    {
      g_free(stm->stream);
      stm->stream = NULL;
    }

  stm->port = 0;

  if(stm->c != NULL)
  {
      gj_connection_free(stm->c);
      stm->c = NULL;
  }
}

stream gj_stream_find_by_id(gchar *id)
{
  gint index = 0;

  if(id == NULL)
    {
      g_warning("%s: id was NULL", __FUNCTION__);
      return NULL;
    }

  for(index=0;index<g_list_length(stream_list);index++)
    {
      stream stm = (stream) g_list_nth_data(stream_list,index);

      if(stm == NULL) continue;
      if(strcmp(stm->id,id) == 0) return stm;
    }
  
  return NULL;
}

stream gj_stream_find_by_unique_id(guint id)
{
  gint index = 0;

  if(id < 1)
    {
      g_warning("%s: unique id:%d, was < 1", __FUNCTION__, id);
      return NULL;
    }

  for(index=0;index<g_list_length(stream_list);index++)
    {
      stream stm = (stream) g_list_nth_data(stream_list,index);

      if(stm == NULL) continue;
      if(stm->unique_id == id) return stm;
    }
  
  return NULL;
}

stream gj_stream_find_by_connection(connection c)
{
  gint index = 0;

  if(c == NULL)
    {
      g_warning("%s: connection was NULL", __FUNCTION__);
      return NULL;
    }

  for(index=0;index<g_list_length(stream_list);index++)
    {
      stream stm = (stream) g_list_nth_data(stream_list,index);

      if(stm == NULL) continue;
      if(stm->c == c) return stm;
    }
  
  return NULL;
}

stream gj_stream_find_by_to(gchar *to)
{
  gint index = 0;

  if(to == NULL)
    {
      g_warning("%s: to was NULL", __FUNCTION__);
      return NULL;
    }

  for(index=0;index<g_list_length(stream_list);index++)
    {
      stream stm = (stream) g_list_nth_data(stream_list,index);

      if(stm == NULL) continue;
      if(strcmp(stm->to,to) == 0) return stm;
    }
  
  return NULL;
}

stream gj_stream_find_by_request_id(gint requestID)
{
  gint index = 0;

  if(requestID <= 0)
    {
      g_warning("%s: id was NULL", __FUNCTION__);
      return NULL;
    }

  for(index=0;index<g_list_length(stream_list);index++)
    {
      stream stm = (stream) g_list_nth_data(stream_list,index);

      if(stm == NULL) continue;
      if(stm->requestID == requestID) return stm;
    }
  
  return NULL;
}

/*
 * member functions
 */

/* sets */
gboolean gj_stream_set_id(stream stm, gchar *id)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return FALSE;
    }

  if(id == NULL)
    {
      g_warning("%s: id was NULL", __FUNCTION__);
      return FALSE;
    }

  if(stm->id != NULL)
    {
      g_free(stm->id);
      stm->id = NULL;
    }
  
  stm->id = g_strdup(id);

  return TRUE;
}

gboolean gj_stream_set_to(stream stm, gchar *to)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return FALSE;
    }

  if(to == NULL)
    {
      g_warning("%s: to was NULL", __FUNCTION__);
      return FALSE;
    }

  if(stm->to != NULL)
    {
      g_free(stm->to);
      stm->to = NULL;
    }
  
  stm->to = g_strdup(to);

  return TRUE;
}

gboolean gj_stream_set_port(stream stm, guint port)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return FALSE;
    }

  if(port < 1024)
    {
      g_warning("%s: port:%d < 1024", __FUNCTION__, port);
      return FALSE;
    }

  stm->port = port;

  return TRUE;
}

gboolean gj_stream_set_stream(stream stm, gchar *stream)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return FALSE;
    }

  if(stream == NULL)
    {
      g_warning("%s: new stream was NULL", __FUNCTION__);
      return FALSE;
    }

  stm->stream = g_strdup(stream);

  return TRUE;
}

gboolean gj_stream_set_xml(stream stm, xmlDocPtr doc)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return FALSE;
    }

  if(doc == NULL)
    {
      g_warning("%s: failed, XML was NULL", __FUNCTION__);
      return FALSE;
    }

  if(stm->xml != NULL)
    {
      xmlFreeDoc(stm->xml);
      stm->xml = NULL;
    }

  if((stm->xml = xmlCopyDoc(doc,TRUE)) == NULL)
    {
      g_warning("%s: xmlCopyDoc failed", __FUNCTION__);
      return FALSE;
    }
 
  return TRUE;
}

void gj_stream_set_is_built(stream stm, gboolean is_built)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return;
    }

  stm->is_built = is_built;

  return;
}

gboolean gj_stream_set_connected_cb(stream stm, streamCallback cb)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("%s: callback was NULL", __FUNCTION__);
      return FALSE;
    }

  stm->cb_connected = cb;

  return TRUE;
}

gboolean gj_stream_set_disconnected_cb(stream stm, streamCallback cb)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("%s: callback was NULL", __FUNCTION__);
      return FALSE;
    }

  stm->cb_disconnected = cb;

  return TRUE;
}

gboolean gj_stream_set_error_cb(stream stm, streamCallback cb)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return FALSE;
    }

  if(cb == NULL)
    {
      g_warning("%s: callback was NULL", __FUNCTION__);
      return FALSE;
    }

  stm->cb_error = cb;

  return TRUE;
}


gboolean gj_stream_set_connection(stream stm, connection c)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return FALSE;
    }

  if(c == NULL)
    {
      g_warning("%s: connection was NULL", __FUNCTION__);
      return FALSE;
    }    

  stm->c = c;

  return TRUE;
}

/* gets */
gchar *gj_stream_get_id(stream stm)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return FALSE;
    }

  return stm->id;
}

gchar *gj_stream_get_to(stream stm)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return FALSE;
    }

  return stm->to;
}

guint gj_stream_get_port(stream stm)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return FALSE;
    }

  return stm->port;
}

guint gj_stream_get_unique_id(stream stm)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return FALSE;
    }

  return stm->unique_id;
}

gchar *gj_stream_get_stream(stream stm)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return FALSE;
    }

  return stm->stream;
}

xmlDocPtr gj_stream_get_xml(stream stm)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return FALSE;
    }

  return stm->xml;
}

gint gj_stream_get_request_id(stream stm)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return -1;
    }

  return stm->requestID;
}

gboolean gj_stream_get_is_built(stream stm)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return FALSE;
    }

  return stm->is_built;
}

connection gj_stream_get_connection(stream stm)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return NULL;
    }

  return stm->c;
}

/* 
 * compile stream
 */
gboolean gj_stream_open(stream stm)
{
  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return FALSE;
    } 

  if(gj_stream_get_is_built(stm) == FALSE)
    {
      g_warning("%s: stream is not built, not continuing...", __FUNCTION__);
      return FALSE;
    }

  if((stm->c = gj_connection_create(gj_stream_get_to(stm), gj_stream_get_port(stm))) == NULL)
  {
      g_warning("%s: connection was NULL", __FUNCTION__);
      return FALSE;
  }

  g_message("%s: connection:0x%.8x created, host:'%s', port:%d", 
	    __FUNCTION__, (gint)stm->c, gj_stream_get_to(stm), gj_stream_get_port(stm));

  /* open tcp connection and make connection */
  gj_connection_set_cb_connect(stm->c, gj_stream_connected_cb);
  gj_connection_set_cb_disconnect(stm->c, gj_stream_disconnected_cb);
  gj_connection_set_cb_error(stm->c, gj_stream_error_cb);
  gj_connection_set_cb_in(stm->c, gj_parser_push_input_from_connection);

  if(gj_connection_connect(stm->c) == FALSE)
    {
      g_warning("%s: failed to connect (locally)", __FUNCTION__);
      return FALSE;
    }

  return TRUE;
}

gboolean gj_stream_close(stream stm)
{
    gchar *msg = NULL;

    if(stm == NULL)
    {
	g_warning("%s: stream was NULL", __FUNCTION__);
	return FALSE;
    }    

    msg = g_strdup_printf("</stream:stream>");
    gj_connection_write(stm->c, msg);

    /* clean up */
    if(msg != NULL) g_free(msg);

    /* free off stream ?? */

    return TRUE;
}

gboolean gj_stream_close_by_to(gchar *to)
{
    stream stm = NULL;

    if(to == NULL)
    {
	g_warning("%s: to was NULL", __FUNCTION__);
	return FALSE;
    }

    if((stm = gj_stream_find_by_to(to)) == NULL)
    {
	g_warning("%s: could not find stream by to:'%s'", __FUNCTION__, to);
	return FALSE;
    }

    return gj_stream_close(stm);
}

/*
 * NOTE: the connected function means we have a socket connection NOT another
 *       stream from the server.  the connected callback is called at the 
 *       'stream receive' function.
 */
void gj_stream_connected_cb(connection c)
{
  gchar *msg = NULL;
  stream stm = NULL;

  if((stm = gj_stream_find_by_connection(c)) == NULL)
    {
      g_warning("%s: could not find stream from connection", __FUNCTION__);
      return;
    }

  /* use stream and introduce ourselves... */
  if((msg = gj_stream_convert_doc_to_str(stm)) == NULL)
    {
      g_warning("%s: failed to convert doc to string, NULL returned", __FUNCTION__);
      return;
    }

  g_message("%s: stream is connected", __FUNCTION__);
  gj_connection_write(c, msg);  

  /* clean up */
  g_free(msg);
}

void gj_stream_disconnected_cb(connection c)
{
  stream stm = NULL;

  if((stm = gj_stream_find_by_connection(c)) == NULL)
  {
      g_warning("%s: could not find stream from connection", __FUNCTION__);
      return;
  }  
  
  /* send up */
  if(stm->cb_disconnected == NULL)
    {
      g_message("%s: no callback for disconnection!", __FUNCTION__);
      return;
    }

  (stm->cb_disconnected)(stm, 0);

  /* free stream */
  gj_stream_free(stm);
}

void gj_stream_error_cb(connection c, GTcpSocketConnectAsyncStatus status)
{
  stream stm = NULL;

  if((stm = gj_stream_find_by_connection(c)) == NULL)
  {
      g_warning("%s: could not find stream from connection", __FUNCTION__);
      return;
  }  
  
  /* send up */
  if(stm->cb_error == NULL)
    {
      g_message("%s: no callback for error!", __FUNCTION__);
      return;
    }

  (stm->cb_error)(stm, status);
}

gboolean gj_stream_build(stream stm)
{
  xmlDocPtr doc = NULL;
  xmlNsPtr ns = NULL;

  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return FALSE;
    }

  if(gj_stream_get_to(stm) == NULL)
    {
      g_warning("%s: 'to' field was NULL", __FUNCTION__);
      return FALSE;
    }

  /* create new doc */
  if((doc = xmlNewDoc((guchar*)"1.0")) == NULL)
    {
      /* failed */
      g_warning("%s: xmlNewDoc failed", __FUNCTION__);
      return FALSE;
    }
  
  ns = xmlNewNs(NULL, (guchar*)"stream", (guchar*)"stream");
  doc->xmlChildrenNode = xmlNewDocNode(doc, ns, (guchar*)"stream", NULL);

  ns = xmlNewNs(doc->xmlChildrenNode, (guchar*)"http://etherx.jabber.org/streams", (guchar*)"stream");
  xmlSetProp(doc->xmlChildrenNode, (guchar*)"to", (guchar*)gj_stream_get_to(stm));
  xmlSetProp(doc->xmlChildrenNode, (guchar*)"xmlns", (guchar*)"jabber:client");
 
  gj_stream_set_xml(stm, doc);
  /* xmlAddChild(doc->xmlChildrenNode,gj_stream_get_xml(stm));*/

  /* set flag to say stream is built */
  gj_stream_set_is_built(stm,TRUE);

  /* free doc ? */

  return TRUE;
}

gchar *gj_stream_convert_doc_to_str(stream stm)
{
  xmlChar *buf;
  gint size = 0;
  gchar *output = NULL;

  if(stm == NULL)
    {
      g_warning("%s: stream was NULL", __FUNCTION__);
      return FALSE;
    }

  if(gj_stream_get_is_built(stm) == FALSE)
    {
      g_warning("%s: stream is not built, not continuing...", __FUNCTION__);
      return FALSE;
    }

  if(gj_stream_get_xml(stm) == NULL)
    {
      g_warning("%s: 'xml' is NULL", __FUNCTION__);
      return FALSE;
    }

  xmlDocDumpMemory(gj_stream_get_xml(stm),&buf,&size); 
/*   buf = g_strdup_printf("<?xml version='1.0'?><stream:stream to='%s' xmlns='jabber:client' xmlns:stream='http://etherx.jabber.org/streams'>", gj_stream_get_to(stm)); */

  if(buf == NULL)
    {
      g_warning("%s: xmlDocDumpMemory failed", __FUNCTION__);
      return NULL;
    }

  /* add a fix for end of xml tag */
  buf[(size-3)] = '>'; 
  buf[(size-2)] = ' '; 

  output = g_strdup_printf("%s",buf);

  /* need to free buf... but free crashes it */

  return output;
}

void gj_stream_receive(connection c, xmlNodePtr node)
{
  stream stm = NULL;

  gint requestID = -1;
  gchar *tmp = NULL;

  /* find stream */
  if((stm = gj_stream_find_by_connection(c)) == NULL)
    {
      g_warning("%s: failed to find stream based on connection:0x%.8x", __FUNCTION__, (gint)c);
      return;
    }
     
  /* set id */
  if((tmp = (gchar*)gj_parser_find_by_attr_and_ret(node, (guchar*)"id")) != NULL)
    gj_stream_set_id(stm,tmp);
  
  /* get call back information */
  if(stm->cb_connected == NULL)
    {
      g_warning("%s: connected callback is NULL", __FUNCTION__);
      return;
    }
  
  /* get requestID - required for callback */
  requestID = gj_stream_get_request_id(stm);
  
  /* call callback function */
  (stm->cb_connected)(stm, requestID);
}

void gj_stream_receive_error(connection c, xmlNodePtr node)
{
  gchar *tmp = NULL;

  if(c == NULL)
  {
      g_warning("%s: connection is NULL", __FUNCTION__);
      return;
  }

  if(node == NULL)
  {
      g_warning("%s: node is NULL", __FUNCTION__);
      return;
  }

  if((tmp = (gchar*)gj_parser_get_content(node)) == NULL)
  {
      g_warning("%s: unknown error", __FUNCTION__);
      return;
  }

  gj_gtk_msg_new(TRUE,
		 GJ_GTK_MSG_BUTTON_CLOSE | GJ_GTK_MSG_ICON_ERROR,
		 (MsgCb)NULL,
		 NULL,
		 _("Stream Error."),
		 "%s",
		 tmp, 
		 NULL);
}

void gj_stream_print_all()
{
  stream stm = NULL;
  gint index = 0;

  g_message("++++++++++++++++++++++ Stream - Print All, Begin. +++++++++++++++++++++");

  for(index=0;index<g_list_length(stream_list);index++)
    {
      stm = (stream) g_list_nth_data(stream_list,index);
      gj_stream_print(stm);
    }

  g_message("++++++++++++++++++++++ Stream - Print All, End.   +++++++++++++++++++++");
  
  return;  
}

void gj_stream_print(stream stm)
{
  if(stm == NULL)
    return;

  g_message("+++++++++++++++++++++++++++ Stream ++++++++++++++++++++++++");
  g_message("++++");
  g_message("++++ request id:'%d'", stm->requestID);
  g_message("++++");
  g_message("++++ cb connected:0x%.8x", (gint)stm->cb_connected);
  g_message("++++ cb disconnect:0x%.8x", (gint)stm->cb_disconnected);
  g_message("++++ cb error:0x%.8x", (gint)stm->cb_error);
  g_message("++++");
  g_message("++++ is build:'%s'", stm->is_built?"yes":"no");
  g_message("++++");
  g_message("++++ id:'%s'", stm->id);
  g_message("++++ to:'%s'", stm->to);
  g_message("++++ stream:%s", stm->stream);
  g_message("++++");
  g_message("++++ unique id:'%d'", stm->unique_id);
  g_message("++++");
  g_message("++++ port:%d", stm->port);
  g_message("++++");
  g_message("++++ connection:0x%.8x", (gint)stm->c);
  g_message("++++");
  g_message("++++ xml document:0x%.8x", (gint)stm->xml);
  g_message("++++");
  g_message("+++++++++++++++++++++++++++ Stream ++++++++++++++++++++++++");
}


#ifdef __cplusplus
}
#endif
